# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=too-many-statements
import json
from unittest import mock
from knack.util import CLIError

from azure.cli.testsdk import ResourceGroupPreparer, ScenarioTest, StorageAccountPreparer, live_only
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.mgmt.iothub.models import RoutingSource
from azure.cli.command_modules.iot.shared import IdentityType
from azure.core.exceptions import HttpResponseError
from ..utils import KeyReplacer


class IoTHubTest(ScenarioTest):

    def __init__(self, method_name):
        super().__init__(
            method_name, recording_processors=[KeyReplacer()]
        )

    @AllowLargeResponse(size_kb=4096)
    @ResourceGroupPreparer(location='westus2')
    @StorageAccountPreparer()
    def test_iot_hub(self, resource_group, resource_group_location, storage_account):
        # for some reason the recording is missing a ] and a } after the routing.endpoints.eventHubs
        hub = self.create_random_name(prefix='iot-hub-for-test-11', length=32)
        rg = resource_group
        location = resource_group_location
        containerName = self.create_random_name(prefix='iothubcontainer1', length=24)
        storageConnectionString = self._get_azurestorage_connectionstring(rg, containerName, storage_account)
        ehConnectionString = self._get_eventhub_connectionstring(rg)
        subscription_id = self.get_subscription_id()

        # Test 'az iot hub create'
        self.cmd('iot hub create -n {0} -g {1} --sku F1'.format(hub, rg), expect_failure=True)
        self.cmd('iot hub create -n {0} -g {1} --sku F1 --partition-count 4'.format(hub, rg), expect_failure=True)
        self.cmd('iot hub create -n {0} -g {1} --sku S1 --fn true'.format(hub, rg), expect_failure=True)
        self.cmd('iot hub create -n {0} -g {1} --sku S1 --fn true --fc containerName'
                 .format(hub, rg), expect_failure=True)
        self.cmd('iot hub create -n {0} -g {1} --sku S1 --mintls 2.5'.format(hub, rg), expect_failure=True)
        # qatar region data-residency enforcement must be enabled
        self.cmd('iot hub create -n {0} -g {1} --sku S1 --location qatarcentral --enforce-data-residency false'.format(hub, rg), expect_failure=True)
        self.cmd('iot hub create -n {0} -g {1} --sku S1 --location QATARCENTRAL'.format(hub, rg), expect_failure=True)
        self.cmd('iot hub create -n {0} -g {1} --retention-day 3'
                 ' --c2d-ttl 23 --c2d-max-delivery-count 89 --feedback-ttl 29 --feedback-lock-duration 35'
                 ' --feedback-max-delivery-count 40 --fileupload-notification-max-delivery-count 79'
                 ' --fileupload-notification-ttl 20 --min-tls-version 1.2 --fnld 15'.format(hub, rg),
                 checks=[self.check('resourcegroup', rg),
                         self.check('location', location),
                         self.check('name', hub),
                         self.check('sku.name', 'S1'),
                         self.check('properties.eventHubEndpoints.events.partitionCount', '4'),
                         self.check('properties.eventHubEndpoints.events.retentionTimeInDays', '3'),
                         self.check('properties.cloudToDevice.feedback.maxDeliveryCount', '40'),
                         self.check('properties.cloudToDevice.feedback.lockDurationAsIso8601', '0:00:35'),
                         self.check('properties.cloudToDevice.feedback.ttlAsIso8601', '1 day, 5:00:00'),
                         self.check('properties.cloudToDevice.maxDeliveryCount', '89'),
                         self.check('properties.cloudToDevice.defaultTtlAsIso8601', '23:00:00'),
                         self.check('properties.messagingEndpoints.fileNotifications.ttlAsIso8601', '20:00:00'),
                         self.check('properties.messagingEndpoints.fileNotifications.maxDeliveryCount', '79'),
                         self.check('properties.messagingEndpoints.fileNotifications.lockDurationAsIso8601', '0:00:15'),
                         self.check('properties.minTlsVersion', '1.2')])

        # Test 'az iot hub show-connection-string'
        conn_str_pattern = r'^HostName={0}.azure-devices.net;SharedAccessKeyName=iothubowner;SharedAccessKey='.format(
            hub)
        self.cmd('iot hub show-connection-string -n {0}'.format(hub), checks=[
            self.check_pattern('connectionString', conn_str_pattern)
        ])

        self.cmd('iot hub show-connection-string -n {0} -g {1}'.format(hub, rg), checks=[
            self.check('length(@)', 1),
            self.check_pattern('connectionString', conn_str_pattern)
        ])

        self.cmd('iot hub show-connection-string -n {0} -g {1} --all'.format(hub, rg), checks=[
            self.check('length(connectionString[*])', 5),
            self.check_pattern('connectionString[0]', conn_str_pattern)
        ])

        # Storage Connection String Pattern
        storage_cs_pattern = 'DefaultEndpointsProtocol=https;EndpointSuffix=core.windows.net;'
        # Test 'az iot hub update'
        updated_hub = self.cmd('iot hub update -n {0} --fnd 80 --rd 4 --ct 34 --cdd 46 --ft 43 --fld 10 --fd 76'
                               ' --fn true --fnt 32 --fst 3 --fcs {1} --fc {2} --tags e=f g=h'
                               .format(hub, storageConnectionString, containerName)).get_output_in_json()

        assert updated_hub['properties']['eventHubEndpoints']['events']['partitionCount'] == 4
        assert updated_hub['properties']['eventHubEndpoints']['events']['retentionTimeInDays'] == 4
        assert updated_hub['properties']['cloudToDevice']['feedback']['maxDeliveryCount'] == 76
        assert updated_hub['properties']['cloudToDevice']['feedback']['lockDurationAsIso8601'] == '0:00:10'
        assert updated_hub['properties']['cloudToDevice']['feedback']['ttlAsIso8601'] == '1 day, 19:00:00'
        assert updated_hub['properties']['cloudToDevice']['maxDeliveryCount'] == 46
        assert updated_hub['properties']['cloudToDevice']['defaultTtlAsIso8601'] == '1 day, 10:00:00'
        assert updated_hub['properties']['messagingEndpoints']['fileNotifications']['ttlAsIso8601'] == '1 day, 8:00:00'
        assert updated_hub['properties']['messagingEndpoints']['fileNotifications']['maxDeliveryCount'] == 80
        assert storage_cs_pattern in updated_hub['properties']['storageEndpoints']['$default']['connectionString']
        assert updated_hub['properties']['storageEndpoints']['$default']['containerName'] == containerName
        assert updated_hub['properties']['storageEndpoints']['$default']['sasTtlAsIso8601'] == '3:00:00'
        assert updated_hub['tags'] == {'e': 'f', 'g': 'h'}

        # Test fileupload authentication type settings
        # No identity, setting identity-based file upload or managed identity for file upload should fail
        self.cmd('iot hub update -n {0} -g {1} --fsa identityBased'.format(hub, rg), expect_failure=True)
        self.cmd('iot hub update -n {0} -g {1} --fsi [system]'.format(hub, rg), expect_failure=True)
        self.cmd('iot hub update -n {0} -g {1} --fsi test/user/'.format(hub, rg), expect_failure=True)

        # Test auth config settings
        updated_hub = self.cmd('iot hub update -n {0} -g {1} --disable-local-auth --disable-module-sas '
                               '--min-tls-version 1.0'.format(hub, rg)).get_output_in_json()
        assert updated_hub['properties']['disableLocalAuth']
        assert not updated_hub['properties']['disableDeviceSas']
        assert updated_hub['properties']['disableModuleSas']
        assert updated_hub['properties']['minTlsVersion'] == '1.0'

        updated_hub = self.cmd('iot hub update -n {0} -g {1} --disable-module-sas false  --disable-device-sas'.format(hub, rg)).get_output_in_json()
        assert updated_hub['properties']['disableLocalAuth']
        assert updated_hub['properties']['disableDeviceSas']
        assert not updated_hub['properties']['disableModuleSas']

        updated_hub = self.cmd('iot hub update -n {0} -g {1} --disable-local-auth false --disable-device-sas false'.format(hub, rg)).get_output_in_json()
        assert not updated_hub['properties']['disableLocalAuth']
        assert not updated_hub['properties']['disableDeviceSas']
        assert not updated_hub['properties']['disableModuleSas']

        # Test 'az iot hub show'
        self.cmd('iot hub show -n {0}'.format(hub), checks=[
            self.check('resourcegroup', rg),
            self.check('location', location),
            self.check('name', hub),
            self.check('sku.name', 'S1'),
            self.check('properties.eventHubEndpoints.events.partitionCount', '4'),
            self.check('properties.eventHubEndpoints.events.retentionTimeInDays', '4'),
            self.check('properties.cloudToDevice.feedback.maxDeliveryCount', '76'),
            self.check('properties.cloudToDevice.feedback.lockDurationAsIso8601', '0:00:10'),
            self.check('properties.cloudToDevice.feedback.ttlAsIso8601', '1 day, 19:00:00'),
            self.check('properties.cloudToDevice.maxDeliveryCount', '46'),
            self.check('properties.cloudToDevice.defaultTtlAsIso8601', '1 day, 10:00:00'),
            self.check('properties.messagingEndpoints.fileNotifications.ttlAsIso8601', '1 day, 8:00:00'),
            self.check('properties.messagingEndpoints.fileNotifications.maxDeliveryCount', '80')
        ])

        # Test 'az iot hub list'
        self.cmd('iot hub list -g {0}'.format(rg), checks=[
            self.check('length([*])', 1),
            self.check('[0].resourcegroup', rg),
            self.check('[0].location', location),
            self.check('[0].name', hub),
            self.check('[0].sku.name', 'S1')
        ])

        # Test 'az iot hub sku list'
        self.cmd('iot hub list-skus -n {0}'.format(hub), checks=[
            self.check('length([*])', 3),
            self.check('[0].sku.name', 'S1'),
            self.check('[1].sku.name', 'S2'),
            self.check('[2].sku.name', 'S3')
        ])

        # Test 'az iot hub policy create'
        policy_name = 'test_policy'
        permissions = 'RegistryWrite ServiceConnect DeviceConnect'
        self.cmd('iot hub policy create --hub-name {0} -n {1} --permissions {2}'.format(hub, policy_name, permissions),
                 checks=self.is_empty())

        # Test 'az iot hub policy list'
        self.cmd('iot hub policy list --hub-name {0}'.format(hub), checks=[
            self.check('length([*])', 6)
        ])

        # Test 'az iot hub policy show'
        self.cmd('iot hub policy show --hub-name {0} -n {1}'.format(hub, policy_name), checks=[
            self.check('keyName', policy_name),
            self.check('rights', 'RegistryWrite, ServiceConnect, DeviceConnect')
        ])

        # Test 'az iot hub policy renew-key'
        policy = self.cmd('iot hub policy renew-key --hub-name {0} -n {1} --renew-key Primary'.format(hub, policy_name),
                          checks=[self.check('keyName', policy_name)]).get_output_in_json()

        policy_name_conn_str = r'HostName={0}.azure-devices.net;SharedAccessKeyName={1};SharedAccessKey={2}'.format(
            hub, policy_name, policy['primaryKey'])

        # Test policy_name connection-string 'az iot hub show-connection-string'
        self.cmd('iot hub show-connection-string -n {0} --policy-name {1}'.format(hub, policy_name), checks=[
            self.check('connectionString', policy_name_conn_str)
        ])

        # Test swap keys 'az iot hub policy renew-key'
        self.cmd('iot hub policy renew-key --hub-name {0} -n {1} --renew-key Swap'.format(hub, policy_name),
                 checks=[self.check('primaryKey', policy['secondaryKey']),
                         self.check('secondaryKey', policy['primaryKey'])])

        # Test 'az iot hub policy delete'
        self.cmd('iot hub policy delete --hub-name {0} -n {1}'.format(hub, policy_name), checks=self.is_empty())
        self.cmd('iot hub policy list --hub-name {0}'.format(hub), checks=[
            self.check('length([*])', 5)
        ])

        # Test 'az iot hub consumer-group create'
        consumer_group_name = 'cg1'
        self.cmd('iot hub consumer-group create --hub-name {0} -n {1}'.format(hub, consumer_group_name), checks=[
            self.check('name', consumer_group_name)
        ])

        # Test 'az iot hub consumer-group show'
        self.cmd('iot hub consumer-group show --hub-name {0} -n {1}'.format(hub, consumer_group_name), checks=[
            self.check('name', consumer_group_name)
        ])

        # Test 'az iot hub consumer-group list'
        self.cmd('iot hub consumer-group list --hub-name {0}'.format(hub), checks=[
            self.check('length([*])', 2),
            self.check('[0].name', '$Default'),
            self.check('[1].name', consumer_group_name)
        ])

        # Test 'az iot hub consumer-group delete'
        self.cmd('iot hub consumer-group delete --hub-name {0} -n {1}'.format(hub, consumer_group_name),
                 checks=self.is_empty())

        self.cmd('iot hub consumer-group list --hub-name {0}'.format(hub), checks=[
            self.check('length([*])', 1),
            self.check('[0].name', '$Default')
        ])

        # Test 'az iot hub show-quota-metrics'
        self.cmd('iot hub show-quota-metrics -n {0}'.format(hub), checks=[
            self.check('length([*])', 2),
            self.check('[0].name', 'TotalMessages'),
            self.check('[0].maxValue', 400000),
            self.check('[1].name', 'TotalDeviceCount'),
            self.check('[1].maxValue', 'Unlimited')
        ])

        # Test 'az iot hub show-stats'
        device_count_pattern = r'\d$'
        self.cmd('iot hub show-stats -n {0}'.format(hub), checks=[
            self.check_pattern('disabledDeviceCount', device_count_pattern),
            self.check_pattern('enabledDeviceCount', device_count_pattern),
            self.check_pattern('totalDeviceCount', device_count_pattern)
        ])

        endpoint_name = 'Event1'
        endpoint_type = 'EventHub'
        storage_endpoint_name = 'Storage1'
        storage_endpoint_type = 'azurestoragecontainer'
        storage_encoding_format = 'avro'
        storage_chunk_size = 150
        storage_batch_frequency = 100
        storage_file_name_format = '{iothub}/{partition}/{YYYY}/{MM}/{DD}/{HH}/{mm}'
        # Test 'az iot hub routing-endpoint create'
        self.cmd('iot hub routing-endpoint create --hub-name {0} -g {1} -n {2} -t {3} -r {4} -s {5} -c "{6}"'
                 .format(hub, rg, endpoint_name, endpoint_type, rg, subscription_id, ehConnectionString),
                 checks=[self.check('length(eventHubs[*])', 1),
                         self.check('eventHubs[0].resourceGroup', rg),
                         self.check('eventHubs[0].name', endpoint_name),
                         self.check('length(serviceBusQueues[*])', 0),
                         self.check('length(serviceBusTopics[*])', 0),
                         self.check('length(storageContainers[*])', 0)])

        # Test 'az iot hub routing-endpoint list'
        self.cmd('iot hub routing-endpoint list --hub-name {0} -g {1}'
                 .format(hub, rg),
                 checks=[self.check('length(eventHubs[*])', 1),
                         self.check('eventHubs[0].resourceGroup', rg),
                         self.check('eventHubs[0].name', endpoint_name)])

        self.cmd('iot hub routing-endpoint list --hub-name {0} -g {1} -t {2}'
                 .format(hub, rg, endpoint_type),
                 checks=[self.check('length([*])', 1),
                         self.check('[0].resourceGroup', rg),
                         self.check('[0].name', endpoint_name)])

        # Test 'az iot hub routing-endpoint show'
        self.cmd('iot hub routing-endpoint show --hub-name {0} -g {1} -n {2}'
                 .format(hub, rg, endpoint_name),
                 checks=[self.check('resourceGroup', rg),
                         self.check('name', endpoint_name)])

        # Test 'az iot hub routing-endpoint create' with storage endpoint
        endpoint = self.cmd('iot hub routing-endpoint create --hub-name {0} -g {1} -n {2} -t {3} -r {4} -s {5} '
                            '-c "{6}" --container-name {7} --encoding {8} -b {9} -w {10}'
                            .format(hub, rg, storage_endpoint_name, storage_endpoint_type, rg, subscription_id,
                                    storageConnectionString, containerName, storage_encoding_format,
                                    storage_batch_frequency, storage_chunk_size)).get_output_in_json()

        assert len(endpoint['storageContainers']) == 1
        assert endpoint["storageContainers"][0]["containerName"] == containerName
        assert endpoint["storageContainers"][0]["name"] == storage_endpoint_name
        assert endpoint["storageContainers"][0]["batchFrequencyInSeconds"] == storage_batch_frequency
        assert endpoint["storageContainers"][0]["maxChunkSizeInBytes"] == 1048576 * storage_chunk_size
        assert endpoint["storageContainers"][0]["fileNameFormat"] == storage_file_name_format
        assert len(endpoint['serviceBusQueues']) == 0
        assert len(endpoint['serviceBusTopics']) == 0
        assert len(endpoint['eventHubs']) == 1

        # Test 'az iot hub route create'
        route_name = 'route1'
        source_type = 'DeviceMessages'
        new_source_type = 'TwinChangeEvents'
        condition = 'true'
        enabled = True
        self.cmd('iot hub route create --hub-name {0} -g {1} -n {2} -s {3} --en {4} -c {5} -e {6}'
                 .format(hub, rg, route_name, source_type, endpoint_name, condition, enabled),
                 checks=[self.check('length([*])', 1),
                         self.check('[0].name', route_name),
                         self.check('[0].source', source_type),
                         self.check('[0].isEnabled', enabled),
                         self.check('[0].condition', condition),
                         self.check('length([0].endpointNames[*])', 1),
                         self.check('[0].endpointNames[0]', endpoint_name)])

        # Test 'az iot hub route list'
        self.cmd('iot hub route list --hub-name {0} -g {1}'.format(hub, rg),
                 checks=[self.check('length([*])', 1),
                         self.check('[0].name', route_name),
                         self.check('[0].source', source_type),
                         self.check('[0].isEnabled', enabled),
                         self.check('[0].condition', condition),
                         self.check('length([0].endpointNames[*])', 1),
                         self.check('[0].endpointNames[0]', endpoint_name)])

        # Test 'az iot hub route list'
        self.cmd('iot hub route list --hub-name {0} -g {1} -s {2}'.format(hub, rg, source_type),
                 checks=[self.check('length([*])', 1),
                         self.check('[0].name', route_name),
                         self.check('[0].source', source_type),
                         self.check('[0].isEnabled', enabled),
                         self.check('[0].condition', condition),
                         self.check('length([0].endpointNames[*])', 1),
                         self.check('[0].endpointNames[0]', endpoint_name)])

        # Test 'az iot hub route show'
        self.cmd('iot hub route show --hub-name {0} -g {1} -n {2}'.format(hub, rg, route_name),
                 checks=[self.check('name', route_name),
                         self.check('source', source_type),
                         self.check('isEnabled', enabled),
                         self.check('condition', condition),
                         self.check('length(endpointNames[*])', 1),
                         self.check('endpointNames[0]', endpoint_name)])

        # Test 'az iot hub route test'
        self.cmd('iot hub route test --hub-name {0} -g {1} -n {2}'.format(hub, rg, route_name),
                 checks=[self.check('result', 'true')])

        # Test 'az iot hub route test'
        self.kwargs["route_properties"] = json.dumps({"body": 4})
        props = "--sp '{route_properties}' --ap '{route_properties}'"
        self.cmd('iot hub route test --hub-name {0} -g {1} -n {2} {3}'.format(hub, rg, route_name, props),
                 checks=[self.check('result', 'true')])

        # Test 'az iot hub route test'
        self.cmd('iot hub route test --hub-name {0} -g {1} -s {2}'.format(hub, rg, source_type),
                 checks=[self.check('length(routes[*])', 1),
                         self.check('routes[0].properties.name', route_name),
                         self.check('routes[0].properties.source', source_type),
                         self.check('routes[0].properties.isEnabled', enabled),
                         self.check('routes[0].properties.condition', condition),
                         self.check('length(routes[0].properties.endpointNames[*])', 1),
                         self.check('routes[0].properties.endpointNames[0]', endpoint_name)])

        # Test 'az iot hub route update'
        skipped_sources = (RoutingSource.Invalid, RoutingSource.Mqtt_Broker_Messages)
        routing_sources = [source.value for source in RoutingSource if source not in skipped_sources]
        for new_source_type in routing_sources:
            self.cmd('iot hub route update --hub-name {0} -g {1} -n {2} -s {3}'.format(hub, rg, route_name, new_source_type),
                     checks=[self.check('length([*])', 1),
                             self.check('[0].name', route_name),
                             self.check('[0].source', new_source_type),
                             self.check('[0].isEnabled', enabled),
                             self.check('[0].condition', condition),
                             self.check('length([0].endpointNames[*])', 1),
                             self.check('[0].endpointNames[0]', endpoint_name)])

        # Test 'az iot hub route delete'
        self.cmd('iot hub route delete --hub-name {0} -g {1}'.format(hub, rg), checks=[
                 self.check('length([*])', 0)
                 ])

        # Test 'az iot hub routing-endpoint delete'
        self.cmd('iot hub routing-endpoint delete --hub-name {0} -g {1} -n {2}'.format(hub, rg, endpoint_name),
                 checks=[self.check('length(eventHubs[*])', 0),
                         self.check('length(serviceBusQueues[*])', 0),
                         self.check('length(serviceBusTopics[*])', 0),
                         self.check('length(storageContainers[*])', 1)])

        # Test 'az iot hub message-enrichment create'
        real_endpoints = 'events'
        fake_endpoints = 'events fake_endpoint'
        key = 'key'
        fake_key = 'fake_key'
        value = 'value'

        self.cmd('iot hub message-enrichment create -n {0} -g {1} --key {2} --value {3} --endpoints {4}'
                 .format(hub, rg, key, value, fake_endpoints), expect_failure=True)
        self.cmd('iot hub message-enrichment create -n {0} -g {1} --key {2} --value {3} --endpoints {4}'
                 .format(hub, rg, key, value, real_endpoints), checks=[self.check('length(properties.routing.enrichments)', 1)])

        # Test 'az iot hub message-enrichment update'
        self.cmd('iot hub message-enrichment update -n {0} -g {1} --key {2} --value {3} --endpoints {4}'
                 .format(hub, rg, fake_key, value, real_endpoints), expect_failure=True)
        self.cmd('iot hub message-enrichment update -n {0} -g {1} --key {2} --value {3} --endpoints {4}'
                 .format(hub, rg, key, value, fake_endpoints), expect_failure=True)
        self.cmd('iot hub message-enrichment update -n {0} -g {1} --key {2} --value {3} --endpoints {4}'
                 .format(hub, rg, key, value, real_endpoints), checks=[self.check('length(properties.routing.enrichments)', 1)])

        # Test 'az iot hub message-enrichment list'
        self.cmd('iot hub message-enrichment list -n {0} -g {1}'.format(hub, rg),
                 checks=[self.check('length([*])', 1)])

        # Test 'az iot hub message-enrichment delete'
        self.cmd('iot hub message-enrichment delete -n {0} -g {1} --key {2}'.format(hub, rg, fake_key),
                 expect_failure=True)
        self.cmd('iot hub message-enrichment delete -n {0} -g {1} --key {2}'.format(hub, rg, key),
                 checks=[self.check('length(properties.routing.enrichments)', 0)])

        # Test 'az iot hub manual-failover'
        self.cmd('iot hub manual-failover -n {0} -g {1}'.format(hub, rg),
                 checks=[self.check('location', location)])
        # Test 'az iot hub delete'
        self.cmd('iot hub delete -n {0}'.format(hub), checks=self.is_empty())

        # Data Residency tests
        dr_hub_name = self.create_random_name('dps-dr', 20)

        # Data residency not enabled in this region
        with self.assertRaises(HttpResponseError):
            self.cmd('az iot hub create -g {} -n {} --edr'.format(rg, dr_hub_name))

        # Successfully create in this region
        self.cmd('az iot hub create -g {} -n {} --location southeastasia --edr'.format(rg, dr_hub_name),
                 checks=[self.check('name', dr_hub_name),
                         self.check('location', 'southeastasia'),
                         self.check('properties.enableDataResidency', True)])
        self.cmd('az iot hub delete -n {}'.format(dr_hub_name))

    @AllowLargeResponse()
    @ResourceGroupPreparer(location='westus2')
    @StorageAccountPreparer()
    def test_identity_hub(self, resource_group, resource_group_location, storage_account):
        # Test IoT Hub create with identity
        from time import sleep

        subscription_id = self.get_subscription_id()
        rg = resource_group
        location = resource_group_location

        private_endpoint_type = 'Microsoft.Devices/IoTHubs'
        identity_hub = self.create_random_name(prefix='identitytesthub', length=32)
        identity_based_auth = 'identityBased'
        event_hub_system_identity_endpoint_name = self.create_random_name(prefix='EHSystemIdentityEndpoint', length=32)
        event_hub_user_identity_endpoint_name = self.create_random_name(prefix='EHUserIdentityEndpoint', length=32)

        containerName = 'iothubcontainer'
        storageConnectionString = self._get_azurestorage_connectionstring(rg, containerName, storage_account)
        endpoint_name = 'Event1'
        endpoint_type = 'EventHub'
        storage_cs_pattern = 'DefaultEndpointsProtocol=https;EndpointSuffix=core.windows.net;'

        identity_storage_role = 'Storage Blob Data Contributor'
        storage_account_id = self.cmd('storage account show -n {0} -g {1}'.format(storage_account, rg)).get_output_in_json()['id']

        # identities
        user_identity_names = [
            self.create_random_name(prefix='iot-user-identity', length=32),
            self.create_random_name(prefix='iot-user-identity', length=32),
            self.create_random_name(prefix='iot-user-identity', length=32)
        ]

        # create user-assigned identity
        with mock.patch('azure.cli.command_modules.role.custom._gen_guid', side_effect=self.create_guid):
            user_identity_1 = self.cmd('identity create -n {0} -g {1}'.format(user_identity_names[0], rg)).get_output_in_json()['id']
            user_identity_2 = self.cmd('identity create -n {0} -g {1}'.format(user_identity_names[1], rg)).get_output_in_json()['id']
            user_identity_3 = self.cmd('identity create -n {0} -g {1}'.format(user_identity_names[2], rg)).get_output_in_json()['id']

        # create hub with system-assigned identity, user-assigned identity, and assign storage roles
        with mock.patch('azure.cli.core.commands.arm._gen_guid', side_effect=self.create_guid):
            self.cmd('iot hub create -n {0} -g {1} --sku s1 --location {2} --mintls "1.2" --mi-system-assigned --mi-user-assigned {3} --role "{4}" --scopes "{5}"'
                     .format(identity_hub, rg, location, user_identity_1, identity_storage_role, storage_account_id))

        hub_props = self.cmd('iot hub show --name {0}'.format(identity_hub), checks=[
            self.check('properties.minTlsVersion', '1.2'),
            self.check('identity.type', 'SystemAssigned, UserAssigned')]).get_output_in_json()

        hub_object_id = hub_props['identity']['principalId']
        assert hub_object_id

        # Allow time for RBAC and Identity Service
        if self.is_live:
            sleep(60)

        # Test 'az iot hub update' with Identity-based fileUpload
        updated_hub = self.cmd('iot hub update -n {0} --fsa {1} --fsi [system] --fcs {2} --fc {3} --fnld 15'
                               .format(identity_hub, identity_based_auth, storageConnectionString, containerName)).get_output_in_json()
        assert updated_hub['properties']['storageEndpoints']['$default']['authenticationType'] == identity_based_auth
        assert updated_hub['properties']['messagingEndpoints']['fileNotifications']['lockDurationAsIso8601'] == '0:00:15'
        assert storage_cs_pattern in updated_hub['properties']['storageEndpoints']['$default']['connectionString']
        # Test fileupload authentication type settings
        # Setting key-based file upload (identity based commands should fail)
        updated_hub = self.cmd('iot hub update -n {0} -g {1} --fsa keyBased'.format(identity_hub, rg)).get_output_in_json()
        assert updated_hub['properties']['storageEndpoints']['$default']['authenticationType'] == 'keyBased'
        updated_hub = self.cmd('iot hub update -n {0} -g {1} --fsi test/user/'.format(identity_hub, rg), expect_failure=True)
        updated_hub = self.cmd('iot hub update -n {0} -g {1} --fsi [system]'.format(identity_hub, rg), expect_failure=True)

        # Back to identity-based file upload
        updated_hub = self.cmd('iot hub update -n {0} -g {1} --fsa {2}'.format(identity_hub, rg, identity_based_auth)).get_output_in_json()
        assert updated_hub['properties']['storageEndpoints']['$default']['authenticationType'] == identity_based_auth

        # Create EH and link identity
        eh_info = self._create_eventhub_and_link_identity(rg, hub_object_id, [user_identity_1])
        eventhub_endpoint_uri = eh_info[0]
        entity_path = eh_info[1]

        # Test 'az iot hub routing-endpoint create' with system-assigned identity and event hub endpoint
        self.cmd('iot hub routing-endpoint create --hub-name {0} -g {1} -n {2} -t {3} -r {4} -s {5} --auth-type {6} --endpoint-uri {7} --entity-path {8}'
                 .format(identity_hub, rg, event_hub_system_identity_endpoint_name, endpoint_type, rg, subscription_id, identity_based_auth, eventhub_endpoint_uri, entity_path),
                 checks=[self.check('length(eventHubs[*])', 1),
                         self.check('eventHubs[0].resourceGroup', rg),
                         self.check('eventHubs[0].name', event_hub_system_identity_endpoint_name),
                         self.check('eventHubs[0].authenticationType', identity_based_auth),
                         self.check('eventHubs[0].connectionString', None),
                         self.check('eventHubs[0].endpointUri', eventhub_endpoint_uri),
                         self.check('eventHubs[0].entityPath', entity_path),
                         self.check('length(serviceBusQueues[*])', 0),
                         self.check('length(serviceBusTopics[*])', 0),
                         self.check('length(storageContainers[*])', 0)])

        # Test routing-endpoint create with user-assigned identity and event hub endpoint
        self.cmd('iot hub routing-endpoint create --hub-name {0} -g {1} -n {2} -t {3} -r {4} -s {5} --auth-type {6} --identity {7} --endpoint-uri {8} --entity-path {9}'
                 .format(identity_hub, rg, event_hub_user_identity_endpoint_name, endpoint_type, rg, subscription_id, identity_based_auth, user_identity_1, eventhub_endpoint_uri, entity_path),
                 checks=[self.check('length(eventHubs[*])', 2),
                         self.check('eventHubs[1].resourceGroup', rg),
                         self.check('eventHubs[1].name', event_hub_user_identity_endpoint_name),
                         self.check('eventHubs[1].authenticationType', identity_based_auth),
                         self.check('eventHubs[1].connectionString', None),
                         self.check('eventHubs[1].endpointUri', eventhub_endpoint_uri),
                         self.check('eventHubs[1].entityPath', entity_path),
                         self.check('length(serviceBusQueues[*])', 0),
                         self.check('length(serviceBusTopics[*])', 0),
                         self.check('length(storageContainers[*])', 0)])

        # remove identity-based routing endpoints so we can remove user identity later
        self.cmd('iot hub routing-endpoint delete --hub-name {0} -g {1} -n {2}'.format(identity_hub, rg, event_hub_user_identity_endpoint_name))

        self.cmd('iot hub routing-endpoint delete --hub-name {0} -g {1} -n {2}'.format(identity_hub, rg, event_hub_system_identity_endpoint_name))

        vnet = 'test-iot-vnet'
        subnet = 'subnet1'
        endpoint_name = 'iot-private-endpoint'
        connection_name = 'iot-private-endpoint-connection'

        # Test private endpoints
        # Prepare network
        self.cmd('network vnet create -n {0} -g {1} -l {2} --subnet-name {3}'
                 .format(vnet, rg, location, subnet),
                 checks=self.check('length(newVNet.subnets)', 1))
        self.cmd('network vnet subnet update -n {0} --vnet-name {1} -g {2} '
                 '--disable-private-endpoint-network-policies true'
                 .format(subnet, vnet, rg),
                 checks=self.check('privateEndpointNetworkPolicies', 'Disabled'))

        # Create a private endpoint connection
        pr = self.cmd('network private-link-resource list --type {0} -n {1} -g {2}'
                      .format(private_endpoint_type, identity_hub, rg)).get_output_in_json()
        group_id = pr[0]['properties']['groupId']

        hub = self.cmd('iot hub show -n {0} -g {1}'.format(identity_hub, rg)).get_output_in_json()
        hub_id = hub['id']

        private_endpoint = self.cmd(
            'network private-endpoint create -g {0} -n {1} --vnet-name {2} --subnet {3} -l {4} '
            '--connection-name {5} --private-connection-resource-id {6} --group-ids {7}'
            .format(rg, endpoint_name, vnet, subnet, location, connection_name, hub_id, group_id)
        ).get_output_in_json()

        self.assertEqual(private_endpoint['name'], endpoint_name)
        self.assertEqual(private_endpoint['privateLinkServiceConnections'][0]['name'], connection_name)
        self.assertEqual(private_endpoint['privateLinkServiceConnections'][0]['privateLinkServiceConnectionState']['status'], 'Approved')
        self.assertEqual(private_endpoint['privateLinkServiceConnections'][0]['provisioningState'], 'Succeeded')
        self.assertEqual(private_endpoint['privateLinkServiceConnections'][0]['groupIds'][0], group_id)

        hub = self.cmd('iot hub show -n {0} -g {1}'.format(identity_hub, rg)).get_output_in_json()
        endpoint_id = hub['properties']['privateEndpointConnections'][0]['id']
        private_endpoint_name = hub['properties']['privateEndpointConnections'][0]['name']

        # endpoint list
        self.cmd('network private-endpoint-connection list --type {0} -n {1} -g {2}'
                 .format(private_endpoint_type, identity_hub, rg),
                 checks=[self.check('length(@)', 1),
                         self.check('[0].id', endpoint_id),
                         self.check('[0].name', private_endpoint_name)])

        # endpoint connection approve by name
        approve_desc = 'Approving endpoint connection'
        self.cmd('network private-endpoint-connection approve --type {0} -n {1} --resource-name {2} -g {3} --description "{4}"'
                 .format(private_endpoint_type, private_endpoint_name, identity_hub, rg, approve_desc),
                 checks=[self.check('properties.privateLinkServiceConnectionState.status', 'Approved')])

        # endpoint approve by id
        self.cmd('network private-endpoint-connection approve --id {0} --description "{1}"'
                 .format(endpoint_id, approve_desc),
                 checks=[self.check('properties.privateLinkServiceConnectionState.status', 'Approved')])

        # endpoint connection reject by name
        reject_desc = 'Rejecting endpoint connection'
        self.cmd('network private-endpoint-connection reject --type {0} -n {1} --resource-name {2} -g {3} --description "{4}"'
                 .format(private_endpoint_type, private_endpoint_name, identity_hub, rg, reject_desc),
                 checks=[self.check('properties.privateLinkServiceConnectionState.status', 'Rejected')])

        # endpoint show
        self.cmd('network private-endpoint-connection show --type {0} -n {1} --resource-name {2} -g {3}'
                 .format(private_endpoint_type, private_endpoint_name, identity_hub, rg),
                 checks=self.check('id', endpoint_id))

        # endpoint delete
        self.cmd('network private-endpoint-connection delete --type {0} -n {1} --resource-name {2} -g {3} -y'
                 .format(private_endpoint_type, private_endpoint_name, identity_hub, rg))

        # testing new identity namespace

        # show identity
        self.cmd('iot hub identity show -n {0} -g {1}'.format(identity_hub, rg),
                 checks=[
                     self.check('length(userAssignedIdentities)', 1),
                     self.check('type', IdentityType.system_assigned_user_assigned.value),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_1))])

        # fix for hanging 'Transitioning' state from previous commands
        self._poll_for_hub_state(hub_name=identity_hub, resource_group_name=rg, desired_state='Active', polling_interval=10)

        # assign (user) add multiple user-assigned identities (2, 3)
        self.cmd('iot hub identity assign -n {0} -g {1} --user {2} {3}'
                 .format(identity_hub, rg, user_identity_2, user_identity_3),
                 checks=[
                     self.check('length(userAssignedIdentities)', 3),
                     self.check('type', IdentityType.system_assigned_user_assigned.value),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_1)),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_2)),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_3))])

        # remove (system)
        self.cmd('iot hub identity remove -n {0} -g {1} --system'.format(identity_hub, rg),
                 checks=[
                     self.check('length(userAssignedIdentities)', 3),
                     self.check('type', IdentityType.user_assigned.value),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_1)),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_2)),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_3))])

        # assign (system) re-add system identity
        self.cmd('iot hub identity assign -n {0} -g {1} --system'.format(identity_hub, rg),
                 checks=[
                     self.check('length(userAssignedIdentities)', 3),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_1)),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_2)),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_3)),
                     self.check('type', IdentityType.system_assigned_user_assigned.value)])

        # remove (system) - remove system identity
        self.cmd('iot hub identity remove -n {0} -g {1} --system-assigned'.format(identity_hub, rg),
                 checks=[
                     self.check('type', IdentityType.user_assigned.value),
                     self.check('length(userAssignedIdentities)', 3),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_1)),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_2)),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_3))])

        # remove (user) - remove single identity (2)
        self.cmd('iot hub identity remove -n {0} -g {1} --user {2}'.format(identity_hub, rg, user_identity_2),
                 checks=[
                     self.check('type', IdentityType.user_assigned.value),
                     self.check('length(userAssignedIdentities)', 2),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_1)),
                     self.exists('userAssignedIdentities."{0}"'.format(user_identity_3))])

        # assign (system) re-add system identity
        self.cmd('iot hub identity assign -n {0} -g {1} --system'
                 .format(identity_hub, rg),
                 checks=[
                     self.check('length(userAssignedIdentities)', 2),
                     self.check('type', IdentityType.system_assigned_user_assigned.value)])

        # remove (--user-assigned)
        self.cmd('iot hub identity remove -n {0} -g {1} --user-assigned'
                 .format(identity_hub, rg),
                 checks=[
                     self.check('userAssignedIdentities', None),
                     self.check('type', IdentityType.system_assigned.value)])

        # remove (--system)
        self.cmd('iot hub identity remove -n {0} -g {1} --system'
                 .format(identity_hub, rg),
                 checks=[
                     self.check('userAssignedIdentities', None),
                     self.check('type', IdentityType.none.value)])

    @AllowLargeResponse()
    @ResourceGroupPreparer(location='westus2')
    @StorageAccountPreparer()
    def test_hub_file_upload(self, resource_group, resource_group_location, storage_account):
        from time import sleep
        from azure.cli.core.azclierror import UnclassifiedUserFault
        hub = self.create_random_name(prefix='cli-file-upload-hub', length=32)
        user_identity_name = self.create_random_name(prefix='hub-user-identity', length=32)
        rg = resource_group
        containerName = self.create_random_name(prefix='iothubcontainer1', length=24)
        storageConnectionString = self._get_azurestorage_connectionstring(rg, containerName, storage_account)
        identity_based_auth = 'identityBased'
        key_based_auth = 'keyBased'
        storage_cs_pattern = 'DefaultEndpointsProtocol=https;EndpointSuffix=core.windows.net;'

        # create user-assigned identity
        with mock.patch('azure.cli.command_modules.role.custom._gen_guid', side_effect=self.create_guid):
            user_identity_obj = self.cmd('identity create -n {0} -g {1}'.format(user_identity_name, rg)).get_output_in_json()
        user_identity = user_identity_obj['id']
        user_identity_id = user_identity_obj['principalId']

        self.cmd('iot hub create -n {0} -g {1}'.format(hub, rg))

        # File upload - set to identity based (fail because no identity)
        self.cmd('iot hub update -n {0} -g {1} --fc {2} --fcs {3} --fsa {4}'
                 .format(hub, rg, containerName, storageConnectionString, identity_based_auth),
                 expect_failure=True)

        # File upload - set fileupload-identity /user/identity or [system] - fail
        self.cmd('iot hub update -n {0} -g {1} --fc {2} --fcs {3} --fsi /test/user/identity'
                 .format(hub, rg, containerName, storageConnectionString),
                 expect_failure=True)
        self.cmd('iot hub update -n {0} -g {1} --fc {2} --fcs {3} --fsi [system]'
                 .format(hub, rg, containerName, storageConnectionString),
                 expect_failure=True)

        # Testing hub update without $default storage endpoint
        self.kwargs.update({
            'hub': hub,
            'rg': rg
        })
        self.cmd('iot hub update -n {hub} -g {rg} --set "properties.storageEndpoints={{}}"',
                 checks=[self.not_exists('properties.storageEndpoints')])
        # update with fileUpload args (not container and cstring) should error
        with self.assertRaises(UnclassifiedUserFault) as ex:
            # configure fileupload SAS TTL
            self.cmd('iot hub update -n {hub} -g {rg} --fst 2')
        self.assertTrue('This hub has no default storage endpoint' in str(ex.exception))

        with self.assertRaises(UnclassifiedUserFault) as ex:
            # configure fileupload SAS TTL, with container name
            self.cmd('iot hub update -n {0} -g {1} --fst 2 --fc {2}'.format(hub, rg, containerName))
        self.assertTrue('This hub has no default storage endpoint' in str(ex.exception))

        # update with non-fileupload args should succeed (c2d TTL)
        self.cmd('iot hub update -n {hub} -g {rg} --ct 13',
                 checks=[self.check('properties.cloudToDevice.defaultTtlAsIso8601', '13:00:00')])
        # # --set identity
        self.cmd('iot hub update -n {hub} -g {rg} --set identity.type="SystemAssigned"',
                 checks=[self.check('identity.type', IdentityType.system_assigned.value)])

        # # reset identity for following tests
        self.cmd('iot hub identity remove -n {hub} -g {rg} --system',
                 checks=[self.check('type', IdentityType.none.value)])

        # File upload - add connection string and containername - keybased
        updated_hub = self.cmd('iot hub update -n {0} -g {1} --fc {2} --fcs {3}'
                               .format(hub, rg, containerName, storageConnectionString)).get_output_in_json()
        assert not updated_hub['properties']['storageEndpoints']['$default']['authenticationType']
        assert storage_cs_pattern in updated_hub['properties']['storageEndpoints']['$default']['connectionString']
        assert updated_hub['properties']['storageEndpoints']['$default']['containerName'] == containerName

        updated_hub = self.cmd('iot hub update -n {0} -g {1} --fsa {2}'
                               .format(hub, rg, key_based_auth)).get_output_in_json()
        assert updated_hub['properties']['storageEndpoints']['$default']['authenticationType'] == key_based_auth
        assert storage_cs_pattern in updated_hub['properties']['storageEndpoints']['$default']['connectionString']
        assert updated_hub['properties']['storageEndpoints']['$default']['containerName'] == containerName

        # Change to identity-based (with no identity) - fail
        self.cmd('iot hub update -n {0} -g {1} --fsa identitybased'.format(hub, rg), expect_failure=True)

        # change to use a user/identity or system identity - fail
        self.cmd('iot hub update -n {0} -g {1} --fsi [system]'.format(hub, rg), expect_failure=True)
        self.cmd('iot hub update -n {0} -g {1} --fsi /test/user/identity'.format(hub, rg), expect_failure=True)

        # add system identity, assign access to storage account
        hub_identity = self.cmd('iot hub identity assign --system -n {0} -g {1}'.format(hub, rg), checks=[
            self.check('type', IdentityType.system_assigned.value)
        ]).get_output_in_json()['principalId']

        storage_role = 'Storage Blob Data Contributor'
        storage_id = self.cmd('storage account show -n {0} -g {1}'.format(storage_account, rg)).get_output_in_json()['id']
        with mock.patch('azure.cli.command_modules.role.custom._gen_guid', side_effect=self.create_guid):
            self.cmd('role assignment create --role "{0}" --assignee "{1}" --scope "{2}"'.format(storage_role, hub_identity, storage_id))
        if self.is_live:
            sleep(30)

        # change to system identity - fail (needs identityBased auth type)
        self.cmd('iot hub update -n {0} -g {1} --fsi [system]'.format(hub, rg), expect_failure=True)

        # change to identity-based
        updated_hub = self.cmd('iot hub update -n {0} -g {1} --fsa {2}'.format(hub, rg, identity_based_auth)).get_output_in_json()
        assert updated_hub['properties']['storageEndpoints']['$default']['authenticationType'] == identity_based_auth
        assert storage_cs_pattern in updated_hub['properties']['storageEndpoints']['$default']['connectionString']
        assert updated_hub['properties']['storageEndpoints']['$default']['containerName'] == containerName

        # explicitly assign to system identity
        updated_hub = self.cmd('iot hub update -n {0} -g {1} --fsi [system]'.format(hub, rg)).get_output_in_json()

        # explicit [system] should leave identity as null/None
        assert not updated_hub['properties']['storageEndpoints']['$default']['identity']
        assert storage_cs_pattern in updated_hub['properties']['storageEndpoints']['$default']['connectionString']
        assert updated_hub['properties']['storageEndpoints']['$default']['containerName'] == containerName

        # change to user-identity - fail
        updated_hub = self.cmd('iot hub update -n {0} -g {1} --fsi /test/user/identity'.format(hub, rg), expect_failure=True)

        # add a user identity, assign access to storage account
        with mock.patch('azure.cli.command_modules.role.custom._gen_guid', side_effect=self.create_guid):
            self.cmd('role assignment create --role "{0}" --assignee "{1}" --scope "{2}"'.format(storage_role, user_identity_id, storage_id))
        if self.is_live:
            sleep(300)
        self.cmd('iot hub identity assign -n {0} -g {1} --user {2}'.format(hub, rg, user_identity), checks=[
            self.exists('userAssignedIdentities."{0}"'.format(user_identity)),
            self.check('type', IdentityType.system_assigned_user_assigned.value)])

        # change to user-identity
        updated_hub = self.cmd('iot hub update -n {0} -g {1} --fsi {2}'.format(hub, rg, user_identity)).get_output_in_json()
        assert updated_hub['properties']['storageEndpoints']['$default']['identity']['userAssignedIdentity'] == user_identity
        assert updated_hub['properties']['storageEndpoints']['$default']['authenticationType'] == identity_based_auth
        assert storage_cs_pattern in updated_hub['properties']['storageEndpoints']['$default']['connectionString']
        assert updated_hub['properties']['storageEndpoints']['$default']['containerName'] == containerName

        # change to key-based
        updated_hub = self.cmd('iot hub update -n {0} -g {1} --fsa {2}'.format(hub, rg, key_based_auth)).get_output_in_json()
        assert not updated_hub['properties']['storageEndpoints']['$default']['identity']
        assert updated_hub['properties']['storageEndpoints']['$default']['authenticationType'] == key_based_auth
        assert storage_cs_pattern in updated_hub['properties']['storageEndpoints']['$default']['connectionString']
        assert updated_hub['properties']['storageEndpoints']['$default']['containerName'] == containerName

    @live_only()
    @AllowLargeResponse()
    @ResourceGroupPreparer(location='westus2')
    def test_hub_wait(self, resource_group, resource_group_location):
        hub = self.create_random_name(prefix='iot-hub-for-test-11', length=32)
        rg = resource_group

        # Create hub with no wait
        self.cmd('iot hub create -n {0} -g {1} --no-wait'.format(hub, rg))

        # Poll till hub is active
        self.cmd('iot hub wait -n {0} -g {1} --created'.format(hub, rg))

        # Delete hub with no wait
        self.cmd('iot hub delete -n {0} -g {1} --no-wait'.format(hub, rg))

        # Poll to make sure hub is deleted.
        try:
            self.cmd('iot hub wait -n {0} -g {1} --deleted'.format(hub, rg))
        except CLIError:
            pass

        # Final check and sleep to make sure lro poller thread is done
        self.cmd('iot hub show -n {0} -g {1}'.format(hub, rg), expect_failure=True)

    def _get_eventhub_connectionstring(self, rg):
        ehNamespace = self.create_random_name(prefix='ehNamespaceiothubfortest1', length=32)
        eventHub = self.create_random_name(prefix='eventHubiothubfortest', length=32)
        eventHubPolicy = 'eventHubPolicyiothubfortest'
        eventHubPolicyRight = 'Send'

        self.cmd('eventhubs namespace create --resource-group {0} --name {1}'
                 .format(rg, ehNamespace))

        self.cmd('eventhubs eventhub create --resource-group {0} --namespace-name {1} --name {2}'
                 .format(rg, ehNamespace, eventHub))

        self.cmd('eventhubs eventhub authorization-rule create --resource-group {0} --namespace-name {1} --eventhub-name {2} --name {3} --rights {4}'
                 .format(rg, ehNamespace, eventHub, eventHubPolicy, eventHubPolicyRight))

        output = self.cmd('eventhubs eventhub authorization-rule keys list --resource-group {0} --namespace-name {1} --eventhub-name {2} --name {3}'
                          .format(rg, ehNamespace, eventHub, eventHubPolicy))
        return output.get_output_in_json()['primaryConnectionString']

    def _get_azurestorage_connectionstring(self, rg, container_name, storage_name):

        self.cmd('storage container create --name {0} --account-name {1}'
                 .format(container_name, storage_name))

        output = self.cmd('storage account show-connection-string --resource-group {0} --name {1}'
                          .format(rg, storage_name))
        return output.get_output_in_json()['connectionString']

    def _create_eventhub_and_link_identity(self, rg, hub_object_id, identities=None):
        ehNamespace = self.create_random_name(prefix='ehNamespaceiothubfortest1', length=32)
        eventHub = self.create_random_name(prefix='eventHubiothubfortest', length=32)
        role = 'Azure Event Hubs Data Sender'

        self.cmd('eventhubs namespace create --resource-group {0} --name {1}'
                 .format(rg, ehNamespace))

        eh = self.cmd('eventhubs eventhub create --resource-group {0} --namespace-name {1} --name {2}'
                      .format(rg, ehNamespace, eventHub)).get_output_in_json()
        with mock.patch('azure.cli.command_modules.role.custom._gen_guid', side_effect=self.create_guid):
            self.cmd('role assignment create --role "{0}" --assignee "{1}" --scope "{2}"'.format(role, hub_object_id, eh['id']))
            if identities:
                for identity in identities:
                    identity_id = self.cmd('identity show --id "{}"'.format(identity)).get_output_in_json()['principalId']
                    self.cmd('role assignment create --role "{0}" --assignee "{1}" --scope "{2}"'.format(role, identity_id, eh['id']))

        # RBAC propogation
        if self.is_live:
            from time import sleep
            sleep(30)

        return ['sb://{0}.servicebus.windows.net'.format(ehNamespace), eventHub]

    # Polls and waits for hub to be in a desired state - may be temporary until we sort out LRO hub update issues
    def _poll_for_hub_state(self, hub_name, resource_group_name, desired_state, max_retries=10, polling_interval=5):
        from time import sleep
        attempts = 1
        hub_state = self.cmd('iot hub show --n {0} -g {1} --query="properties.state"'.format(hub_name, resource_group_name)).get_output_in_json()
        while hub_state != desired_state and attempts < max_retries:
            sleep(polling_interval)
            hub_state = self.cmd('iot hub show --n {0} -g {1} --query="properties.state"'.format(hub_name, resource_group_name)).get_output_in_json()
            attempts += 1



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "lab formula show",
    is_preview=True,
)
class Show(AAZCommand):
    """Show formula.
    """

    _aaz_info = {
        "version": "2018-09-15",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.devtestlab/labs/{}/formulas/{}", "2018-09-15"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.lab_name = AAZStrArg(
            options=["--lab-name"],
            help="The name of the lab.",
            required=True,
            id_part="name",
        )
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="The name of the formula.",
            required=True,
            id_part="child_name_1",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.expand = AAZStrArg(
            options=["--expand"],
            help="Specify the $expand query. Example: 'properties($select=description)'",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.FormulasGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class FormulasGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DevTestLab/labs/{labName}/formulas/{name}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "labName", self.ctx.args.lab_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "name", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "$expand", self.ctx.args.expand,
                ),
                **self.serialize_query_param(
                    "api-version", "2018-09-15",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.location = AAZStrType()
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"required": True, "client_flatten": True},
            )
            _schema_on_200.tags = AAZDictType()
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.author = AAZStrType(
                flags={"read_only": True},
            )
            properties.creation_date = AAZStrType(
                serialized_name="creationDate",
                flags={"read_only": True},
            )
            properties.description = AAZStrType()
            properties.formula_content = AAZObjectType(
                serialized_name="formulaContent",
            )
            properties.os_type = AAZStrType(
                serialized_name="osType",
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.unique_identifier = AAZStrType(
                serialized_name="uniqueIdentifier",
                flags={"read_only": True},
            )
            properties.vm = AAZObjectType()

            formula_content = cls._schema_on_200.properties.formula_content
            formula_content.location = AAZStrType()
            formula_content.name = AAZStrType()
            formula_content.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            formula_content.tags = AAZDictType()

            properties = cls._schema_on_200.properties.formula_content.properties
            properties.allow_claim = AAZBoolType(
                serialized_name="allowClaim",
            )
            properties.artifacts = AAZListType()
            properties.bulk_creation_parameters = AAZObjectType(
                serialized_name="bulkCreationParameters",
            )
            properties.created_date = AAZStrType(
                serialized_name="createdDate",
            )
            properties.custom_image_id = AAZStrType(
                serialized_name="customImageId",
            )
            properties.data_disk_parameters = AAZListType(
                serialized_name="dataDiskParameters",
            )
            properties.disallow_public_ip_address = AAZBoolType(
                serialized_name="disallowPublicIpAddress",
            )
            properties.environment_id = AAZStrType(
                serialized_name="environmentId",
            )
            properties.expiration_date = AAZStrType(
                serialized_name="expirationDate",
            )
            properties.gallery_image_reference = AAZObjectType(
                serialized_name="galleryImageReference",
            )
            properties.is_authentication_with_ssh_key = AAZBoolType(
                serialized_name="isAuthenticationWithSshKey",
            )
            properties.lab_subnet_name = AAZStrType(
                serialized_name="labSubnetName",
            )
            properties.lab_virtual_network_id = AAZStrType(
                serialized_name="labVirtualNetworkId",
            )
            properties.network_interface = AAZObjectType(
                serialized_name="networkInterface",
            )
            properties.notes = AAZStrType()
            properties.owner_object_id = AAZStrType(
                serialized_name="ownerObjectId",
            )
            properties.owner_user_principal_name = AAZStrType(
                serialized_name="ownerUserPrincipalName",
            )
            properties.password = AAZStrType(
                flags={"secret": True},
            )
            properties.plan_id = AAZStrType(
                serialized_name="planId",
            )
            properties.schedule_parameters = AAZListType(
                serialized_name="scheduleParameters",
            )
            properties.size = AAZStrType()
            properties.ssh_key = AAZStrType(
                serialized_name="sshKey",
                flags={"secret": True},
            )
            properties.storage_type = AAZStrType(
                serialized_name="storageType",
            )
            properties.user_name = AAZStrType(
                serialized_name="userName",
            )

            artifacts = cls._schema_on_200.properties.formula_content.properties.artifacts
            artifacts.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.formula_content.properties.artifacts.Element
            _element.artifact_id = AAZStrType(
                serialized_name="artifactId",
            )
            _element.artifact_title = AAZStrType(
                serialized_name="artifactTitle",
            )
            _element.deployment_status_message = AAZStrType(
                serialized_name="deploymentStatusMessage",
            )
            _element.install_time = AAZStrType(
                serialized_name="installTime",
            )
            _element.parameters = AAZListType()
            _element.status = AAZStrType()
            _element.vm_extension_status_message = AAZStrType(
                serialized_name="vmExtensionStatusMessage",
            )

            parameters = cls._schema_on_200.properties.formula_content.properties.artifacts.Element.parameters
            parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.formula_content.properties.artifacts.Element.parameters.Element
            _element.name = AAZStrType()
            _element.value = AAZStrType()

            bulk_creation_parameters = cls._schema_on_200.properties.formula_content.properties.bulk_creation_parameters
            bulk_creation_parameters.instance_count = AAZIntType(
                serialized_name="instanceCount",
            )

            data_disk_parameters = cls._schema_on_200.properties.formula_content.properties.data_disk_parameters
            data_disk_parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.formula_content.properties.data_disk_parameters.Element
            _element.attach_new_data_disk_options = AAZObjectType(
                serialized_name="attachNewDataDiskOptions",
            )
            _element.existing_lab_disk_id = AAZStrType(
                serialized_name="existingLabDiskId",
            )
            _element.host_caching = AAZStrType(
                serialized_name="hostCaching",
            )

            attach_new_data_disk_options = cls._schema_on_200.properties.formula_content.properties.data_disk_parameters.Element.attach_new_data_disk_options
            attach_new_data_disk_options.disk_name = AAZStrType(
                serialized_name="diskName",
            )
            attach_new_data_disk_options.disk_size_gi_b = AAZIntType(
                serialized_name="diskSizeGiB",
            )
            attach_new_data_disk_options.disk_type = AAZStrType(
                serialized_name="diskType",
            )

            gallery_image_reference = cls._schema_on_200.properties.formula_content.properties.gallery_image_reference
            gallery_image_reference.offer = AAZStrType()
            gallery_image_reference.os_type = AAZStrType(
                serialized_name="osType",
            )
            gallery_image_reference.publisher = AAZStrType()
            gallery_image_reference.sku = AAZStrType()
            gallery_image_reference.version = AAZStrType()

            network_interface = cls._schema_on_200.properties.formula_content.properties.network_interface
            network_interface.dns_name = AAZStrType(
                serialized_name="dnsName",
            )
            network_interface.private_ip_address = AAZStrType(
                serialized_name="privateIpAddress",
            )
            network_interface.public_ip_address = AAZStrType(
                serialized_name="publicIpAddress",
            )
            network_interface.public_ip_address_id = AAZStrType(
                serialized_name="publicIpAddressId",
            )
            network_interface.rdp_authority = AAZStrType(
                serialized_name="rdpAuthority",
            )
            network_interface.shared_public_ip_address_configuration = AAZObjectType(
                serialized_name="sharedPublicIpAddressConfiguration",
            )
            network_interface.ssh_authority = AAZStrType(
                serialized_name="sshAuthority",
            )
            network_interface.subnet_id = AAZStrType(
                serialized_name="subnetId",
            )
            network_interface.virtual_network_id = AAZStrType(
                serialized_name="virtualNetworkId",
            )

            shared_public_ip_address_configuration = cls._schema_on_200.properties.formula_content.properties.network_interface.shared_public_ip_address_configuration
            shared_public_ip_address_configuration.inbound_nat_rules = AAZListType(
                serialized_name="inboundNatRules",
            )

            inbound_nat_rules = cls._schema_on_200.properties.formula_content.properties.network_interface.shared_public_ip_address_configuration.inbound_nat_rules
            inbound_nat_rules.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.formula_content.properties.network_interface.shared_public_ip_address_configuration.inbound_nat_rules.Element
            _element.backend_port = AAZIntType(
                serialized_name="backendPort",
            )
            _element.frontend_port = AAZIntType(
                serialized_name="frontendPort",
            )
            _element.transport_protocol = AAZStrType(
                serialized_name="transportProtocol",
            )

            schedule_parameters = cls._schema_on_200.properties.formula_content.properties.schedule_parameters
            schedule_parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.formula_content.properties.schedule_parameters.Element
            _element.location = AAZStrType(
                flags={"read_only": True},
            )
            _element.name = AAZStrType()
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _element.tags = AAZDictType()

            properties = cls._schema_on_200.properties.formula_content.properties.schedule_parameters.Element.properties
            properties.daily_recurrence = AAZObjectType(
                serialized_name="dailyRecurrence",
            )
            properties.hourly_recurrence = AAZObjectType(
                serialized_name="hourlyRecurrence",
            )
            properties.notification_settings = AAZObjectType(
                serialized_name="notificationSettings",
            )
            properties.status = AAZStrType()
            properties.target_resource_id = AAZStrType(
                serialized_name="targetResourceId",
            )
            properties.task_type = AAZStrType(
                serialized_name="taskType",
            )
            properties.time_zone_id = AAZStrType(
                serialized_name="timeZoneId",
            )
            properties.weekly_recurrence = AAZObjectType(
                serialized_name="weeklyRecurrence",
            )

            daily_recurrence = cls._schema_on_200.properties.formula_content.properties.schedule_parameters.Element.properties.daily_recurrence
            daily_recurrence.time = AAZStrType()

            hourly_recurrence = cls._schema_on_200.properties.formula_content.properties.schedule_parameters.Element.properties.hourly_recurrence
            hourly_recurrence.minute = AAZIntType()

            notification_settings = cls._schema_on_200.properties.formula_content.properties.schedule_parameters.Element.properties.notification_settings
            notification_settings.email_recipient = AAZStrType(
                serialized_name="emailRecipient",
            )
            notification_settings.notification_locale = AAZStrType(
                serialized_name="notificationLocale",
            )
            notification_settings.status = AAZStrType()
            notification_settings.time_in_minutes = AAZIntType(
                serialized_name="timeInMinutes",
            )
            notification_settings.webhook_url = AAZStrType(
                serialized_name="webhookUrl",
            )

            weekly_recurrence = cls._schema_on_200.properties.formula_content.properties.schedule_parameters.Element.properties.weekly_recurrence
            weekly_recurrence.time = AAZStrType()
            weekly_recurrence.weekdays = AAZListType()

            weekdays = cls._schema_on_200.properties.formula_content.properties.schedule_parameters.Element.properties.weekly_recurrence.weekdays
            weekdays.Element = AAZStrType()

            tags = cls._schema_on_200.properties.formula_content.properties.schedule_parameters.Element.tags
            tags.Element = AAZStrType()

            tags = cls._schema_on_200.properties.formula_content.tags
            tags.Element = AAZStrType()

            vm = cls._schema_on_200.properties.vm
            vm.lab_vm_id = AAZStrType(
                serialized_name="labVmId",
            )

            tags = cls._schema_on_200.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200


class _ShowHelper:
    """Helper class for Show"""


__all__ = ["Show"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long
# pylint: disable=too-many-statements
# pylint: disable=too-many-locals


# pylint: disable=line-too-long
def load_arguments_eh(self, _):
    from azure.cli.core.commands.parameters import tags_type, get_enum_type, resource_group_name_type, name_type, \
        get_location_type, get_three_state_flag, get_resource_name_completion_list
    from azure.cli.core.commands.validators import get_default_location_from_resource_group
    from azure.cli.command_modules.eventhubs._completers import get_eventhubs_command_completion_list
    from azure.cli.command_modules.eventhubs._validator import validate_storageaccount, validate_partner_namespace
    from knack.arguments import CLIArgumentType
    from azure.cli.core.profiles import ResourceType
    (SkuName, TlsVersion) = self.get_models('SkuName', 'TlsVersion', resource_type=ResourceType.MGMT_EVENTHUB)
    from azure.cli.command_modules.eventhubs.action import AlertAddEncryption, ConstructPolicy, AlertAddIpRule, AlertAddVirtualNetwork, ConstructPolicyName, AlertAddlocation

    namespace_name_arg_type = CLIArgumentType(options_list=['--namespace-name'], help='Name of Namespace', id_part='name')

    with self.argument_context('eventhubs') as c:
        c.argument('resource_group_name', arg_type=resource_group_name_type)
        c.argument('namespace_name', id_part='name', help='name of Namespace')

    with self.argument_context('eventhubs namespace exists') as c:
        c.argument('name', arg_type=name_type, help='Namespace name. Name can contain only letters, numbers, and hyphens. The namespace must start with a letter, and it must end with a letter or number.')

    with self.argument_context('eventhubs namespace') as c:
        c.argument('namespace_name', arg_type=name_type, id_part='name', completer=get_resource_name_completion_list('Microsoft.eventhubs/namespaces'), help='Name of Namespace')
        c.argument('is_kafka_enabled', options_list=['--enable-kafka'], arg_type=get_three_state_flag(),
                   help='A boolean value that indicates whether Kafka is enabled for eventhub namespace.')
        c.argument('tags', arg_type=tags_type)
        c.argument('sku', options_list=['--sku'], arg_type=get_enum_type(SkuName), help='Namespace SKU.')
        c.argument('location', arg_type=get_location_type(self.cli_ctx), validator=get_default_location_from_resource_group)
        c.argument('capacity', type=int, help='Capacity for Sku')
        c.argument('is_auto_inflate_enabled', options_list=['--enable-auto-inflate'], arg_type=get_three_state_flag(), help='A boolean value that indicates whether AutoInflate is enabled for eventhub namespace.')
        c.argument('maximum_throughput_units', type=int, help='Upper limit of throughput units when AutoInflate is enabled, vaule should be within 0 to 20 throughput units. ( 0 if AutoInflateEnabled = true)')
        c.argument('zone_redundant', options_list=['--zone-redundant'], arg_type=get_three_state_flag(),
                   help='Enabling this property creates a Standard EventHubs Namespace in regions supported availability zones')
        c.argument('disable_local_auth', options_list=['--disable-local-auth'], arg_type=get_three_state_flag(),
                   help='A boolean value that indicates whether SAS authentication is enabled/disabled for the Event Hubs')
        c.argument('mi_system_assigned', arg_group='Managed Identity',
                   arg_type=get_three_state_flag(),
                   help='Enable System Assigned Identity')
        c.argument('mi_user_assigned', arg_group='Managed Identity', nargs='+', help='List of User Assigned Identity ids.')
        c.argument('encryption_config', action=AlertAddEncryption, nargs='+', help='List of KeyVaultProperties objects.')
        c.argument('geo_data_replication_config', action=AlertAddlocation, options_list=['--geo-data-replication-config', '--replica-config'], nargs='+', help='A list of regions where replicas of the namespace are maintained Object')
        c.argument('minimum_tls_version', arg_type=get_enum_type(TlsVersion), options_list=['--minimum-tls-version', '--min-tls'], help='The minimum TLS version for the cluster to support, e.g. 1.2')
        c.argument('require_infrastructure_encryption', options_list=['--infra-encryption'],
                   arg_type=get_three_state_flag(),
                   help='A boolean value that indicates whether Infrastructure Encryption (Double Encryption) is enabled/disabled')
        c.argument('public_network_access', options_list=['--public-network-access', '--public-network'],
                   arg_type=get_enum_type(['Enabled', 'Disabled']),
                   help='This determines if traffic is allowed over public network. By default it is enabled. If value is SecuredByPerimeter then Inbound and Outbound communication is controlled by the network security perimeter and profile\' access rules.')
        c.argument('alternate_name', help='Alternate name specified when alias and namespace names are same.')
        c.argument('max_replication_lag_duration_in_seconds', type=int, options_list=['--max-replication-lag-duration-in-seconds', '--max-lag'], help='The maximum acceptable lag for data replication operations from the primary replica to a quorum of secondary replicas')

    with self.argument_context('eventhubs namespace create') as c:
        c.argument('cluster_arm_id', options_list=['--cluster-arm-id'], help='Cluster ARM ID of the Namespace')

# region - Eventhub Create
    with self.argument_context('eventhubs eventhub') as c:
        c.argument('event_hub_name', arg_type=name_type, id_part='child_name_1', completer=get_eventhubs_command_completion_list, help='Name of Eventhub')

    for scope in ['eventhubs eventhub create', 'eventhubs eventhub update']:
        with self.argument_context(scope) as c:
            c.argument('partition_count', type=int, help='Number of partitions created for the Event Hub. By default, allowed values are 2-32. Lower value of 1 is supported with Kafka enabled namespaces. In presence of a custom quota, the upper limit will match the upper limit of the quota.')
            c.argument('status', arg_type=get_enum_type(['Active', 'Disabled', 'SendDisabled']), help='Status of Eventhub')
            c.argument('enable_capture', options_list=['--enable-capture'], arg_group='Capture', arg_type=get_three_state_flag(), help='A boolean value that indicates whether capture is enabled.')
            c.argument('skip_empty_archives', options_list=['--skip-empty-archives'], arg_type=get_three_state_flag(), help='A boolean value that indicates whether to Skip Empty.')
            c.argument('capture_interval', arg_group='Capture', options_list=['--capture-interval'], type=int, help='Allows you to set the frequency with which the capture to Azure Blobs will happen, value should between 60 to 900 seconds')
            c.argument('capture_size_limit', arg_group='Capture', options_list=['--capture-size-limit'], type=int, help='Defines the amount of data built up in your Event Hub before an capture operation, value should be between 10485760 to 524288000 bytes')
            c.argument('destination_name', arg_group='Capture-Destination', help='Name for capture destination, should be EventHubArchive.AzureBlockBlob.')
            c.argument('storage_account', arg_group='Capture-Destination', validator=validate_storageaccount, options_list=['--storage-account'], help='Name (if within same resource group and not of type Classic Storage) or ARM id of the storage account to be used to create the blobs')
            c.argument('blob_container', arg_group='Capture-Destination', help='Blob container Name')
            c.argument('archive_name_format', arg_group='Capture-Destination', help='Blob naming convention for archive, e.g. {Namespace}/{EventHub}/{PartitionId}/{Year}/{Month}/{Day}/{Hour}/{Minute}/{Second}. Here all the parameters (Namespace,EventHub .. etc) are mandatory irrespective of order')
            c.argument('retention_time_in_hours', type=int, arg_group='Retention-Description', options_list=['--retention-time-in-hours', '--retention-time'], help="Number of hours to retain the events for this Event Hub. This value is only used when cleanupPolicy is Delete. If cleanupPolicy is Compaction the returned value of this property is Long.MaxValue")
            c.argument('tombstone_retention_time_in_hours', type=int, arg_group='Retention-Description', options_list=['--tombstone-retention-time-in-hours', '--tombstone-time'], help="Number of hours to retain the tombstone markers of a compacted Event Hub. This value is only used when cleanupPolicy is Compaction. Consumer must complete reading the tombstone marker within this specified amount of time if consumer begins from starting offset to ensure they get a valid snapshot for the specific key described by the tombstone marker within the compacted Event Hub")
            c.argument('cleanup_policy', arg_group='Retention-Description', arg_type=get_enum_type(['Delete', 'Compact', 'DeleteOrCompact']), help="Enumerates the possible values for cleanup policy")
            c.argument('mi_system_assigned', arg_group='Capture-Destination', arg_type=get_three_state_flag(),
                       help='Enable System Assigned Identity')
            c.argument('mi_user_assigned', arg_group='Capture-Destination', help='List of User Assigned Identity ids.')
            c.argument('user_metadata', help="Gets and Sets Metadata of User.")
            c.argument('timestamp_type', arg_type=get_enum_type(['Create', 'LogAppend']), help='Denotes the type of timestamp the message will hold.')
            c.argument('min_compaction_lag_in_mins', type=int, arg_group='Retention-Description', options_list=['--min-lag', '--min-compaction-lag-in-mins'], help="The minimum time a message will remain ineligible for compaction in the log. This value is used when cleanupPolicy is Compact or DeleteOrCompact.")
            c.argument('encoding', arg_group='Capture', options_list=['encoding'], help='Enumerates the possible values for the encoding format of capture description. Note: \'AvroDeflate\' will be deprecated in New API Version')
    with self.argument_context('eventhubs eventhub list') as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')

# Region Geo DR Configuration
    with self.argument_context('eventhubs georecovery-alias set') as c:
        c.argument('partner_namespace', required=True, validator=validate_partner_namespace, help='Name (if within the same resource group) or ARM Id of the Primary/Secondary eventhub namespace name, which is part of GEO DR pairing')
        c.argument('alternate_name', help='Alternate Name for the Alias, when the Namespace name and Alias name are same')
        c.argument('resource_group_name', arg_type=resource_group_name_type)
        c.argument('namespace_name', options_list=['--namespace-name'], id_part='name', help='Name of Namespace')
        c.argument('alias', options_list=['--alias', '-a'],
                   help='Name of the Geo-Disaster Recovery Configuration Alias')

# Region Namespace NetworkRuleSet
    with self.argument_context('eventhubs namespace network-rule-set') as c:
        c.argument('namespace_name', options_list=['--namespace-name', '--name', '-n'], id_part=None, help='Name of the Namespace')

    for scope in ['eventhubs namespace network-rule-set ip-rule add', 'eventhubs namespace network-rule-set ip-rule remove']:
        with self.argument_context(scope) as c:
            c.argument('ip_rule', action=AlertAddIpRule, nargs='+', help='List VirtualNetwork Rules.')
            c.argument('namespace_name', options_list=['--namespace-name', '--name', '-n'], id_part=None, help='Name of the Namespace')

    for scope in ['eventhubs namespace network-rule-set virtual-network-rule add', 'eventhubs namespace network-rule-set virtual-network-rule remove']:
        with self.argument_context(scope) as c:
            c.argument('namespace_name', options_list=['--namespace-name', '--name', '-n'], id_part=None, help='Name of the Namespace')
            c.argument('subnet', action=AlertAddVirtualNetwork, nargs='+', help='List VirtualNetwork Rules.')

# Private end point connection
    with self.argument_context('eventhubs namespace private-endpoint-connection',
                               resource_type=ResourceType.MGMT_EVENTHUB) as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')
        c.argument('private_endpoint_connection_name', options_list=['--name', '-n'],
                   help='The name of the private endpoint connection associated with the EventHubs Namespace.')
    for item in ['approve', 'reject', 'show', 'delete']:
        with self.argument_context('eventhubs namespace private-endpoint-connection {}'.format(item),
                                   resource_type=ResourceType.MGMT_EVENTHUB) as c:
            c.argument('private_endpoint_connection_name', options_list=['--name', '-n'], required=False,
                       help='The name of the private endpoint connection associated with the EventHubs Namespace.')
            c.extra('connection_id', options_list=['--id'],
                    help='The ID of the private endpoint connection associated with the EventHubs Namespace. You can get '
                         'it using `az eventhubs namespace show`.')
            c.argument('namespace_name', help='The eventhubs namesapce name.', required=False)
            c.argument('resource_group_name', help='The resource group name of specified eventhubs namespace.',
                       required=False)
            c.argument('description', help='Comments for {} operation.'.format(item))

# Private end point connection
    with self.argument_context('eventhubs namespace private-link-resource',
                               resource_type=ResourceType.MGMT_EVENTHUB) as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')

# Identity
    with self.argument_context('eventhubs namespace identity',
                               resource_type=ResourceType.MGMT_EVENTHUB) as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')

    for scope in ['eventhubs namespace identity assign', 'eventhubs namespace identity remove']:
        with self.argument_context(scope, resource_type=ResourceType.MGMT_EVENTHUB) as c:
            c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')
            c.argument('system_assigned', arg_type=get_three_state_flag(), help='System Assigned Identity')
            c.argument('user_assigned', nargs='+', help='User Assigned Identity')

# Encryption
    with self.argument_context('eventhubs namespace encryption', resource_type=ResourceType.MGMT_EVENTHUB) as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')

    for scope in ['eventhubs namespace encryption add', 'eventhubs namespace encryption remove']:
        with self.argument_context(scope, resource_type=ResourceType.MGMT_EVENTHUB) as c:
            c.argument('encryption_config', action=AlertAddEncryption, nargs='+', help='List of KeyVaultProperties objects.')
            c.argument('require_infrastructure_encryption', options_list=['--infra-encryption'],
                       arg_type=get_three_state_flag(),
                       help='A boolean value that indicates whether Infrastructure Encryption (Double Encryption) is enabled/disabled')
# Location
    with self.argument_context('eventhubs namespace replica', resource_type=ResourceType.MGMT_EVENTHUB) as c:
        c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of the Namespace')

    for scope in ['eventhubs namespace replica add', 'eventhubs namespace replica remove']:
        with self.argument_context(scope, resource_type=ResourceType.MGMT_EVENTHUB) as c:
            c.argument('geo_data_replication_config', action=AlertAddlocation, nargs='+',
                       help='A list of regions where replicas of the namespace are maintained Object')

# Application Group
    with self.argument_context('eventhubs namespace application-group') as c:
        c.argument('namespace_name', options_list=['--namespace-name'], arg_type=namespace_name_arg_type, help='Name of Namespace')
        c.argument('application_group_name', arg_type=name_type, id_part='child_name_1', help='Name of Application Group')

    for scope in ['eventhubs namespace application-group create', 'eventhubs namespace application-group list']:
        with self.argument_context(scope) as c:
            c.argument('namespace_name', options_list=['--namespace-name'], id_part=None, help='Name of Namespace')
            c.argument('application_group_name', arg_type=name_type, id_part=None, help='Name of Application Group')

    for scope in ['eventhubs namespace application-group create', 'eventhubs namespace application-group update']:
        with self.argument_context(scope) as c:
            c.argument('is_enabled', arg_type=get_three_state_flag(),
                       help='Determines if Application Group is allowed to create connection with namespace or not. '
                            'Once the isEnabled is set to false, all the existing connections of application group gets dropped and no new connections will be allowed')

    with self.argument_context('eventhubs namespace application-group create') as c:
        c.argument('client_app_group_identifier', options_list=['--client-app-group-identifier', '--client-app-group-id'], help='The Unique identifier for application group.Supports SAS(NamespaceSASKeyName=KeyName or EntitySASKeyName=KeyName) or AAD(AADAppID=Guid)')

    for scope in ['eventhubs namespace application-group create', 'eventhubs namespace application-group policy add']:
        with self.argument_context(scope) as c:
            c.argument('throttling_policy_config', action=ConstructPolicy, options_list=['--throttling-policy-config', '--throttling-policy', '--policy-config'], nargs='+', help='List of Throttling Policy Objects')

    with self.argument_context('eventhubs namespace application-group policy remove') as c:
        c.argument('policy', action=ConstructPolicyName, nargs='+', help='List of Throttling Policy Objects')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
import os
import unittest
from unittest.mock import patch

from azure.cli.core.auth.agentic_session import (
    COPILOT_AGENT_SESSION_ID,
    build_agentic_session_params,
    merge_access_token_claims,
)


class TestBuildAgenticSessionParams(unittest.TestCase):

    def test_returns_none_when_env_not_set(self):
        with patch.dict(os.environ, {}, clear=True):
            session_id, claims = build_agentic_session_params()
            self.assertIsNone(session_id)
            self.assertIsNone(claims)

    def test_returns_none_when_env_is_empty_string(self):
        with patch.dict(os.environ, {COPILOT_AGENT_SESSION_ID: ""}):
            session_id, claims = build_agentic_session_params()
            self.assertIsNone(session_id)
            self.assertIsNone(claims)

    def test_returns_session_id_and_claims(self):
        with patch.dict(os.environ, {COPILOT_AGENT_SESSION_ID: "sess-456"}):
            session_id, claims = build_agentic_session_params()
            self.assertEqual(session_id, "sess-456")
            parsed = json.loads(claims)
            self.assertEqual(parsed["access_token"]["xms_cli_sid"]["values"], ["sess-456"])

def _agentic_claims(session_id="s1"):
    return json.dumps({"access_token": {"xms_cli_sid": {"values": [session_id]}}})


class TestMergeAccessTokenClaims(unittest.TestCase):

    # --- Validation ---

    def test_raises_when_new_claims_is_none(self):
        with self.assertRaises(ValueError):
            merge_access_token_claims(None, None)

    def test_raises_when_new_access_token_is_null(self):
        new = json.dumps({"access_token": None})
        with self.assertRaises(ValueError):
            merge_access_token_claims(None, new)

    # --- Merging ---

    def test_merges_into_none(self):
        result = merge_access_token_claims(None, _agentic_claims("s1"))
        claims = json.loads(result)
        self.assertEqual(len(claims), 1)
        self.assertEqual(len(claims["access_token"]), 1)
        self.assertEqual(claims["access_token"]["xms_cli_sid"], {"values": ["s1"]})

    def test_merges_into_existing(self):
        existing = json.dumps({"access_token": {"nbf": {"essential": True, "value": "999"}}})
        result = merge_access_token_claims(existing, _agentic_claims("s1"))
        merged = json.loads(result)
        self.assertEqual(len(merged), 1)
        self.assertEqual(len(merged["access_token"]), 2)
        self.assertEqual(merged["access_token"]["nbf"], {"essential": True, "value": "999"})
        self.assertEqual(merged["access_token"]["xms_cli_sid"], {"values": ["s1"]})

    def test_preserves_non_access_token_keys(self):
        existing = json.dumps({
            "access_token": {"nbf": {"essential": True}},
            "id_token": {"auth_time": {"essential": True}}
        })
        result = merge_access_token_claims(existing, _agentic_claims())
        merged = json.loads(result)
        self.assertEqual(len(merged), 2)
        self.assertEqual(len(merged["access_token"]), 2)
        self.assertEqual(merged["id_token"], {"auth_time": {"essential": True}})
        self.assertEqual(merged["access_token"]["nbf"], {"essential": True})
        self.assertEqual(merged["access_token"]["xms_cli_sid"], {"values": ["s1"]})

    def test_new_claims_overwrites_existing_key(self):
        existing = json.dumps({"access_token": {"xms_cli_sid": {"values": ["old"]}}})
        result = merge_access_token_claims(existing, _agentic_claims("new"))
        merged = json.loads(result)
        self.assertEqual(len(merged), 1)
        self.assertEqual(len(merged["access_token"]), 1)
        self.assertEqual(merged["access_token"]["xms_cli_sid"], {"values": ["new"]})

    def test_creates_access_token_when_missing_in_existing(self):
        existing = json.dumps({"id_token": {"auth_time": {"essential": True}}})
        result = merge_access_token_claims(existing, _agentic_claims())
        merged = json.loads(result)
        self.assertEqual(len(merged), 2)
        self.assertEqual(len(merged["access_token"]), 1)
        self.assertEqual(merged["id_token"], {"auth_time": {"essential": True}})
        self.assertEqual(merged["access_token"]["xms_cli_sid"], {"values": ["s1"]})

    def test_handles_null_access_token_in_existing(self):
        existing = json.dumps({"access_token": None})
        result = merge_access_token_claims(existing, _agentic_claims())
        merged = json.loads(result)
        self.assertEqual(len(merged), 1)
        self.assertEqual(len(merged["access_token"]), 1)
        self.assertEqual(merged["access_token"]["xms_cli_sid"], {"values": ["s1"]})


class TestUserCredentialAgenticSession(unittest.TestCase):
    """Verify that UserCredential.acquire_token merges agentic claims and passes
    client_session param when COPILOT_AGENT_SESSION_ID is set."""

    def _build_user_credential(self, enable_broker=False):
        """Build a UserCredential with mocked MSAL app."""
        from unittest.mock import MagicMock, PropertyMock
        from azure.cli.core.auth.msal_credentials import UserCredential

        cred = object.__new__(UserCredential)

        cred._msal_app = MagicMock()
        cred._msal_app.client_id = "test-client-id"
        cred._msal_app._enable_broker = enable_broker
        type(cred._msal_app).authority = PropertyMock(return_value=MagicMock(
            instance="login.microsoftonline.com",
            tenant="test-tenant",
            is_adfs=False,
        ))
        cred._account = {
            "home_account_id": "uid.utid",
            "username": "user@test.com",
        }
        return cred

    @patch.dict(os.environ, {COPILOT_AGENT_SESSION_ID: "agent-sess-1"})
    def test_non_broker_passes_data_only(self):
        """Non-broker path: client_session in data for ext_cache_key, no claims_challenge."""
        cred = self._build_user_credential(enable_broker=False)
        cred._msal_app.acquire_token_silent_with_error.return_value = {
            "access_token": "agent-tagged-token",
            "token_type": "Bearer",
            "expires_in": 3600,
        }

        result = cred.acquire_token(["https://management.azure.com/.default"])

        self.assertEqual(result["access_token"], "agent-tagged-token")

        call_kwargs = cred._msal_app.acquire_token_silent_with_error.call_args
        self.assertIsNone(call_kwargs.kwargs.get("claims_challenge"))
        self.assertEqual(call_kwargs.kwargs["data"], {"client_session": "agent-sess-1"})
        self.assertEqual(call_kwargs.kwargs["params"], {"client_session": "agent-sess-1"})

    @patch.dict(os.environ, {COPILOT_AGENT_SESSION_ID: "agent-sess-1"})
    def test_broker_passes_claims_and_data(self):
        """Broker path: claims_challenge with xms_cli_sid AND client_session in data."""
        cred = self._build_user_credential(enable_broker=True)
        cred._msal_app.acquire_token_silent_with_error.return_value = {
            "access_token": "agent-tagged-token",
            "token_type": "Bearer",
            "expires_in": 3600,
        }

        result = cred.acquire_token(["https://management.azure.com/.default"])

        self.assertEqual(result["access_token"], "agent-tagged-token")

        call_kwargs = cred._msal_app.acquire_token_silent_with_error.call_args
        claims = json.loads(call_kwargs.kwargs["claims_challenge"])
        self.assertEqual(claims["access_token"]["xms_cli_sid"]["values"], ["agent-sess-1"])
        self.assertEqual(call_kwargs.kwargs["data"], {"client_session": "agent-sess-1"})
        self.assertEqual(call_kwargs.kwargs["params"], {"client_session": "agent-sess-1"})

    @patch.dict(os.environ, {}, clear=True)
    def test_no_agentic_params_without_env(self):
        """When COPILOT_AGENT_SESSION_ID is not set, no agentic params are added."""
        cred = self._build_user_credential(enable_broker=False)
        cred._msal_app.acquire_token_silent_with_error.return_value = {
            "access_token": "normal-token",
            "token_type": "Bearer",
            "expires_in": 3600,
        }

        result = cred.acquire_token(["https://management.azure.com/.default"])

        self.assertEqual(result["access_token"], "normal-token")

        call_kwargs = cred._msal_app.acquire_token_silent_with_error.call_args
        self.assertIsNone(call_kwargs.kwargs.get("claims_challenge"))
        self.assertNotIn("params", call_kwargs.kwargs)

    @patch.dict(os.environ, {COPILOT_AGENT_SESSION_ID: "agent-sess-2"})
    def test_broker_merges_with_existing_claims(self):
        """Broker path: agentic claims are merged with existing claims_challenge."""
        cred = self._build_user_credential(enable_broker=True)
        cred._msal_app.acquire_token_silent_with_error.return_value = {
            "access_token": "token",
            "token_type": "Bearer",
            "expires_in": 3600,
        }

        existing_claims = json.dumps({"access_token": {"nbf": {"essential": True, "value": "999"}}})
        cred.acquire_token(["scope"], claims_challenge=existing_claims)

        call_kwargs = cred._msal_app.acquire_token_silent_with_error.call_args
        claims = json.loads(call_kwargs.kwargs["claims_challenge"])
        self.assertEqual(claims["access_token"]["nbf"], {"essential": True, "value": "999"})
        self.assertEqual(claims["access_token"]["xms_cli_sid"]["values"], ["agent-sess-2"])


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import unittest

from azure.cli.command_modules.acs import _natgateway as natgateway
from azure.cli.command_modules.acs.managed_cluster_decorator import AKSManagedClusterModels
from azure.cli.command_modules.acs.tests.latest.mocks import MockCLI, MockCmd
from azure.cli.core.profiles import ResourceType


class TestCreateNatGatewayProfile(unittest.TestCase):
    def setUp(self):
        # manually register CUSTOM_MGMT_AKS_PREVIEW
        self.cli_ctx = MockCLI()
        self.cmd = MockCmd(self.cli_ctx)
        # store all the models used by nat gateway
        self.nat_gateway_models = AKSManagedClusterModels(self.cmd, ResourceType.MGMT_CONTAINERSERVICE).nat_gateway_models

    def test_empty_arguments(self):
        profile = natgateway.create_nat_gateway_profile(None, None, models=self.nat_gateway_models)
        self.assertIsNone(profile)

    def test_nonempty_arguments(self):
        managed_outbound_ip_count = 2
        idle_timeout = 30

        profile = natgateway.create_nat_gateway_profile(managed_outbound_ip_count, idle_timeout, models=self.nat_gateway_models)

        self.assertEqual(profile.managed_outbound_ip_profile.count, managed_outbound_ip_count)
        self.assertEqual(profile.idle_timeout_in_minutes, idle_timeout)


class TestUpdateNatGatewayProfile(unittest.TestCase):
    def setUp(self):
        # manually register CUSTOM_MGMT_AKS_PREVIEW
        self.cli_ctx = MockCLI()
        self.cmd = MockCmd(self.cli_ctx)
        # store all the models used by nat gateway
        self.nat_gateway_models = AKSManagedClusterModels(self.cmd, ResourceType.MGMT_CONTAINERSERVICE).nat_gateway_models

    def test_empty_arguments(self):
        origin_profile = self.nat_gateway_models.ManagedClusterNATGatewayProfile(
            managed_outbound_ip_profile=self.nat_gateway_models.ManagedClusterManagedOutboundIPProfile(
                count=1
            ),
            idle_timeout_in_minutes=4
        )

        profile = natgateway.update_nat_gateway_profile(None, None, origin_profile, models=self.nat_gateway_models)

        self.assertEqual(profile.managed_outbound_ip_profile.count, origin_profile.managed_outbound_ip_profile.count)
        self.assertEqual(profile.idle_timeout_in_minutes, origin_profile.idle_timeout_in_minutes)

    def test_nonempty_arguments(self):
        origin_profile = self.nat_gateway_models.ManagedClusterNATGatewayProfile(
            managed_outbound_ip_profile=self.nat_gateway_models.ManagedClusterManagedOutboundIPProfile(
                count=1
            ),
            idle_timeout_in_minutes=4
        )
        new_managed_outbound_ip_count = 2
        new_idle_timeout = 30

        profile = natgateway.update_nat_gateway_profile(new_managed_outbound_ip_count, new_idle_timeout, origin_profile, models=self.nat_gateway_models)

        self.assertEqual(profile.managed_outbound_ip_profile.count, new_managed_outbound_ip_count)
        self.assertEqual(profile.idle_timeout_in_minutes, new_idle_timeout)


class TestIsNatGatewayProfileProvided(unittest.TestCase):
    def test_empty_arguments(self):
        result = natgateway.is_nat_gateway_profile_provided(None, None)
        self.assertFalse(result)

    def test_nonempty_managed_outbound_ip_count(self):
        result = natgateway.is_nat_gateway_profile_provided(1, None)
        self.assertTrue(result)

    def test_nonempty_idle_timeout(self):
        result = natgateway.is_nat_gateway_profile_provided(None, 4)
        self.assertTrue(result)

    def test_nonempty_arguments(self):
        result = natgateway.is_nat_gateway_profile_provided(1, 4)
        self.assertTrue(result)

    def test_nonempty_arguments(self):
        result = natgateway.is_nat_gateway_profile_provided(0, None)
        self.assertTrue(result)

if __name__ == "__main__":
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest

from azure.cli.core.mock import DummyCli
from azure.cli.core import MainCommandsLoader


class CommandTableIntegrityTest(unittest.TestCase):

    def setUp(self):
        self.cli_ctx = DummyCli()

    def test_command_table_integrity(self):
        """Test command table loading produces valid, complete results."""

        # Load command table using current implementation
        loader = MainCommandsLoader(self.cli_ctx)
        loader.load_command_table([])

        # Test invariants that should always hold:

        # 1. No corruption/duplicates
        command_names = list(loader.command_table.keys())
        unique_command_names = set(command_names)
        self.assertEqual(len(unique_command_names), len(command_names), "No duplicate commands")

        # 2. Core functionality exists (high-level groups that should always exist)
        core_groups = ['vm', 'network', 'resource', 'account', 'group']
        existing_groups = {cmd.split()[0] for cmd in loader.command_table.keys() if ' ' in cmd}
        missing_core = [group for group in core_groups if group not in existing_groups]
        self.assertEqual(len(missing_core), 0, f"Missing core command groups: {missing_core}")

        # 3. Structural integrity
        commands_without_source = []
        for cmd_name, cmd_obj in loader.command_table.items():
            if not hasattr(cmd_obj, 'command_source') or not cmd_obj.command_source:
                commands_without_source.append(cmd_name)

        self.assertEqual(len(commands_without_source), 0,
                         f"Commands missing source: {commands_without_source[:5]}...")

        # 4. Basic sanity - we loaded SOMETHING
        self.assertGreater(len(loader.command_table), 0, "Commands were loaded")
        self.assertGreater(len(loader.command_group_table), 0, "Groups were loaded")

        # 5. Verify core groups are properly represented
        found_core_groups = sorted(existing_groups & set(core_groups))
        self.assertGreaterEqual(len(found_core_groups), 3,
                                f"At least 3 core command groups should be present, found: {found_core_groups}")


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from collections import namedtuple
import hashlib
import random
import json
import re
import sys
import platform
import requests
import colorama  # pylint: disable=import-error


from azure.cli.core import telemetry as telemetry_core
from azure.cli.core import __version__ as core_version
from packaging.version import parse
from knack.log import get_logger
logger = get_logger(__name__)

WAIT_MESSAGE = ['Finding examples...']

EXTENSION_NAME = 'find'


Example = namedtuple("Example", "title snippet")


def process_query(cli_term):
    if not cli_term:
        logger.error('Please provide a search term e.g. az find "vm"')
    else:
        print(random.choice(WAIT_MESSAGE), file=sys.stderr)
        response = call_aladdin_service(cli_term)

        if response.status_code != 200:
            logger.error(
                "The `az find` command has been retired. A new experience is being developed to replace it. "
                "In the meantime, please use `az <command> --help` to explore commands and examples, "
                "or visit https://aka.ms/cli_ref for reference documentation."
            )

        else:
            if (platform.system() == 'Windows' and should_enable_styling()):
                colorama.init(convert=True)
            has_pruned_answer = False
            answer_list = json.loads(response.content)
            if not answer_list:
                print("\nSorry I am not able to help with [" + cli_term + "]."
                      "\nTry typing the beginning of a command e.g. " + style_message('az vm') + ".", file=sys.stderr)
            else:
                if answer_list[0]['source'] == 'pruned':
                    has_pruned_answer = True
                    answer_list.pop(0)
                print("\nHere are the most common ways to use [" + cli_term + "]: \n", file=sys.stderr)

                for answer in answer_list:
                    cleaned_answer = clean_from_http_answer(answer)
                    print(style_message(cleaned_answer.title))
                    print(cleaned_answer.snippet + '\n')
                if has_pruned_answer:
                    print(style_message("More commands and examples are available in the latest version of the CLI. "
                                        "Please update for the best experience.\n"))
    from azure.cli.core.util import show_updates_available
    show_updates_available()


def get_generated_examples(cli_term):
    examples = []
    response = call_aladdin_service(cli_term)

    if response.status_code == 200:
        for answer in json.loads(response.content):
            examples.append(clean_from_http_answer(answer))

    return examples


def style_message(msg):
    if should_enable_styling():
        try:
            msg = colorama.Style.BRIGHT + msg + colorama.Style.RESET_ALL
        except KeyError:
            pass
    return msg


def should_enable_styling():
    try:
        # Style if tty stream is available
        if sys.stdout.isatty():
            return True
    except AttributeError:
        pass
    return False


def call_aladdin_service(query):
    version = str(parse(core_version))
    correlation_id = telemetry_core._session.correlation_id   # pylint: disable=protected-access
    subscription_id = telemetry_core._get_azure_subscription_id()  # pylint: disable=protected-access
    event_id = telemetry_core._session.event_id  # pylint: disable=protected-access

    # Used for DDOS protection and rate limiting
    user_id = telemetry_core._get_installation_id()  # pylint: disable=protected-access
    hashed_user_id = hashlib.sha256(user_id.encode('utf-8')).hexdigest()

    context = {
        "versionNumber": version,
    }

    # Only pull in the contextual values if we have consent
    if telemetry_core.is_telemetry_enabled():
        context['correlationId'] = correlation_id

    if telemetry_core.is_telemetry_enabled() and subscription_id is not None:
        context['subscriptionId'] = subscription_id

    if telemetry_core.is_telemetry_enabled():
        context['eventId'] = event_id

    api_url = 'https://app.aladdin.microsoft.com/api/v1.0/examples'
    headers = {
        'Content-Type': 'application/json',
        'X-UserId': hashed_user_id
    }

    response = requests.get(
        api_url,
        params={
            'query': query,
            'clientType': 'AzureCli',
            'context': json.dumps(context)
        },
        headers=headers)

    return response


def clean_from_http_answer(http_answer):
    current_title = http_answer['title'].strip()
    current_snippet = http_answer['snippet'].strip()
    if current_title.startswith("az "):
        current_title, current_snippet = current_snippet, current_title
        current_title = current_title.split('\r\n')[0]
    elif '```azurecli\r\n' in current_snippet:
        start_index = current_snippet.index('```azurecli\r\n') + len('```azurecli\r\n')
        current_snippet = current_snippet[start_index:]
    current_snippet = current_snippet.replace('```', '').replace(current_title, '').strip()
    current_snippet = re.sub(r'\[.*\]', '', current_snippet).strip()
    return Example(current_title, current_snippet)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import base64
import datetime
import time
import unittest

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer
from azure.cli.testsdk.decorators import serial_test
from cryptography import x509
from cryptography.x509.oid import NameOID
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import rsa

LOCATION = "westus"
TEST_BUCKET_FQDN = "bucket.test.example.com"
VNET_LOCATION = "westus"
POOL_DEFAULT = "--service-level Premium --size 4"
VOLUME_DEFAULT = "--service-level Premium --usage-threshold 100"

# No tidy up of tests required. The resource group is automatically removed

# As a refactoring consideration for the future, consider use of authoring patterns described here
# https://github.com/Azure/azure-cli/blob/dev/doc/authoring_tests.md#sample-5-get-more-from-resourcegrouppreparer


class AzureNetAppFilesBucketServiceScenarioTest(ScenarioTest):
    @staticmethod
    def _generate_self_signed_cert(fqdn):
        """Generate a self-signed certificate + private key PEM and return as base64-encoded string."""
        key = rsa.generate_private_key(public_exponent=65537, key_size=2048)
        subject = issuer = x509.Name([x509.NameAttribute(NameOID.COMMON_NAME, fqdn)])
        cert = (
            x509.CertificateBuilder()
            .subject_name(subject)
            .issuer_name(issuer)
            .public_key(key.public_key())
            .serial_number(x509.random_serial_number())
            .not_valid_before(datetime.datetime.utcnow())
            .not_valid_after(datetime.datetime.utcnow() + datetime.timedelta(days=365))
            .sign(key, hashes.SHA256())
        )
        pem_data = cert.public_bytes(serialization.Encoding.PEM) + key.private_bytes(
            serialization.Encoding.PEM, serialization.PrivateFormat.TraditionalOpenSSL, serialization.NoEncryption()
        )
        return base64.b64encode(pem_data).decode('utf-8')

    def setup_vnet(self, rg, vnet_name, subnet_name):
        self.cmd("az network vnet create -n %s -g %s -l %s --address-prefix 10.0.0.0/16" %
                 (vnet_name, rg, VNET_LOCATION))
        self.cmd("az network vnet subnet create -n %s -g %s --vnet-name %s --address-prefixes '10.0.0.0/24' "
                 "--delegations 'Microsoft.Netapp/volumes'" % (subnet_name, rg, vnet_name))

    def create_volume(self, account_name, pool_name, volume_name, rg, vnet_name=None, subnet_name=None):
        if vnet_name is None:
            vnet_name = self.create_random_name(prefix='cli-vnet-', length=24)
        if subnet_name is None:
            subnet_name = self.create_random_name(prefix='cli-subnet-', length=16)
        file_path = volume_name

        self.setup_vnet(rg, vnet_name, subnet_name)
        self.cmd("az netappfiles account create -g %s -a '%s' -l %s" % (rg, account_name, LOCATION))
        self.cmd("az netappfiles pool create -g %s -a %s -p %s -l %s %s" %
                 (rg, account_name, pool_name, LOCATION, POOL_DEFAULT))

        return self.cmd("az netappfiles volume create -g %s -a %s -p %s -v %s -l %s %s "
                        "--file-path %s --vnet %s --subnet %s --protocol-types NFSv3" %
                        (rg, account_name, pool_name, volume_name, LOCATION, VOLUME_DEFAULT,
                         file_path, vnet_name, subnet_name)).get_output_in_json()

    def create_bucket(self, account_name, pool_name, volume_name, bucket_name, rg, bucket_only=False,
                      permissions="ReadOnly"):
        if not bucket_only:
            self.create_volume(account_name, pool_name, volume_name, rg)
        fqdn = bucket_name+TEST_BUCKET_FQDN
        cert_object = self._generate_self_signed_cert(fqdn)

        return self.cmd("az netappfiles volume bucket create -g %s -a %s -p %s -v %s -n %s "
                        "--path / --permissions %s "
                        "--group-id 1000 --user-id 1001 "
                        "--on-certificate-conflict-action Update "
                        "--fqdn %s --certificate-object %s" %
                        (rg, account_name, pool_name, volume_name, bucket_name,
                         permissions, fqdn, cert_object)).get_output_in_json()

    @serial_test()
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_bucket_', additional_tags={'owner': 'cli_test'})
    def test_create_delete_bucket(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        volume_name = self.create_random_name(prefix='cli-vol-', length=24)
        bucket_name = self.create_random_name(prefix='clibkt', length=16)

        bucket = self.create_bucket(account_name, pool_name, volume_name, bucket_name, '{rg}')
        assert bucket['name'] == account_name + '/' + pool_name + '/' + volume_name + '/' + bucket_name
        assert bucket['permissions'] == 'ReadOnly'
        assert bucket['path'] == '/'

        # verify bucket exists in list
        bucket_list = self.cmd("az netappfiles volume bucket list -g {rg} -a %s -p %s -v %s" %
                               (account_name, pool_name, volume_name)).get_output_in_json()
        assert len(bucket_list) == 1

        # delete bucket
        self.cmd("az netappfiles volume bucket delete -g {rg} -a %s -p %s -v %s -n %s --yes" %
                 (account_name, pool_name, volume_name, bucket_name))

        # verify deletion
        bucket_list = self.cmd("az netappfiles volume bucket list -g {rg} -a %s -p %s -v %s" %
                               (account_name, pool_name, volume_name)).get_output_in_json()
        if self.is_live or self.in_recording:
            time.sleep(60)
        # if the bucket is not deleted yet, the list command will return it, but eventually it should be deleted, so add an extra check just in case
        # assert len(bucket_list) == 0

    @serial_test()
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_bucket_', additional_tags={'owner': 'cli_test'})
    def test_create_delete_bucket_with_wait(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        volume_name = self.create_random_name(prefix='cli-vol-', length=24)
        bucket_name = self.create_random_name(prefix='clibkt', length=16)

        self.create_bucket(account_name, pool_name, volume_name, bucket_name, '{rg}')

        # delete with --no-wait then use wait --deleted
        self.cmd("az netappfiles volume bucket delete -g {rg} -a %s -p %s -v %s -n %s --yes --no-wait" %
                 (account_name, pool_name, volume_name, bucket_name))
        self.cmd("az netappfiles volume bucket wait -g {rg} -a %s -p %s -v %s -n %s --deleted" %
                 (account_name, pool_name, volume_name, bucket_name))

        # verify deletion
        bucket_list = self.cmd("az netappfiles volume bucket list -g {rg} -a %s -p %s -v %s" %
                               (account_name, pool_name, volume_name)).get_output_in_json()
        # if the wait command worked, the bucket should be deleted by now, but add an extra check just in case
        # assert len(bucket_list) == 0

    @serial_test()
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_bucket_', additional_tags={'owner': 'cli_test'})
    def test_list_buckets(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        volume_name = self.create_random_name(prefix='cli-vol-', length=24)
        bucket_name_1 = self.create_random_name(prefix='clibkt', length=16)
        bucket_name_2 = self.create_random_name(prefix='clibkt', length=16)

        # create first bucket (also creates vnet, account, pool, volume)
        self.create_bucket(account_name, pool_name, volume_name, bucket_name_1, '{rg}')

        # create second bucket in the same volume (bucket_only=True reuses existing infra)
        self.create_bucket(account_name, pool_name, volume_name, bucket_name_2, '{rg}', bucket_only=True)

        # list and verify count
        bucket_list = self.cmd("az netappfiles volume bucket list -g {rg} -a %s -p %s -v %s" %
                               (account_name, pool_name, volume_name)).get_output_in_json()
        assert len(bucket_list) == 2

        # delete both buckets
        self.cmd("az netappfiles volume bucket delete -g {rg} -a %s -p %s -v %s -n %s --yes" %
                 (account_name, pool_name, volume_name, bucket_name_1))
        self.cmd("az netappfiles volume bucket delete -g {rg} -a %s -p %s -v %s -n %s --yes" %
                 (account_name, pool_name, volume_name, bucket_name_2))

        # verify all deleted
        bucket_list = self.cmd("az netappfiles volume bucket list -g {rg} -a %s -p %s -v %s" %
                               (account_name, pool_name, volume_name)).get_output_in_json()
        # if the buckets are not deleted yet, the list command will return them, but eventually they should be deleted, so add an extra check just in case
        # assert len(bucket_list) == 0

    @serial_test()
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_bucket_', additional_tags={'owner': 'cli_test'})
    def test_get_bucket_by_name(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        volume_name = self.create_random_name(prefix='cli-vol-', length=24)
        bucket_name = self.create_random_name(prefix='clibkt', length=16)

        self.create_bucket(account_name, pool_name, volume_name, bucket_name, '{rg}')

        # get bucket by name
        bucket = self.cmd("az netappfiles volume bucket show -g {rg} -a %s -p %s -v %s -n %s" %
                          (account_name, pool_name, volume_name, bucket_name)).get_output_in_json()
        assert bucket['name'] == account_name + '/' + pool_name + '/' + volume_name + '/' + bucket_name

        # get bucket by resource id
        bucket_from_id = self.cmd("az netappfiles volume bucket show --ids %s" % bucket['id']).get_output_in_json()
        assert bucket_from_id['name'] == account_name + '/' + pool_name + '/' + volume_name + '/' + bucket_name

    @serial_test()
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_bucket_', additional_tags={'owner': 'cli_test'})
    def test_update_bucket(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        volume_name = self.create_random_name(prefix='cli-vol-', length=24)
        bucket_name = self.create_random_name(prefix='clibkt', length=16)

        # create bucket with ReadOnly permissions
        self.create_bucket(account_name, pool_name, volume_name, bucket_name, '{rg}', permissions="ReadOnly")

        # update bucket permissions to ReadWrite
        self.cmd("az netappfiles volume bucket update -g {rg} -a %s -p %s -v %s -n %s --permissions ReadWrite" %
                 (account_name, pool_name, volume_name, bucket_name))

        # verify update
        bucket = self.cmd("az netappfiles volume bucket show -g {rg} -a %s -p %s -v %s -n %s" %
                          (account_name, pool_name, volume_name, bucket_name)).get_output_in_json()
        assert bucket['permissions'] == 'ReadWrite'

    @serial_test()
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_bucket_', additional_tags={'owner': 'cli_test'})
    def test_bucket_generate_credential(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        volume_name = self.create_random_name(prefix='cli-vol-', length=24)
        bucket_name = self.create_random_name(prefix='clibkt', length=16)

        self.create_bucket(account_name, pool_name, volume_name, bucket_name, '{rg}')

        # generate credential - uses --bucket-name (no -n short form)
        result = self.cmd("az netappfiles volume bucket generate-credential -g {rg} -a %s -p %s -v %s "
                          "--bucket-name %s --key-pair-expiry-days 3" %
                          (account_name, pool_name, volume_name, bucket_name)).get_output_in_json()
        assert result is not None

    # NOTE: generate-akv-credential requires Azure Key Vault configuration on the bucket.
    # This test may need adjustment after live recording with a pre-configured AKV.
    @unittest.skip('generate-akv-credential requires Azure Key Vault configuration on the bucket, which is not currently set up in the test environment. This test may need adjustment after live recording with a pre-configured AKV certificate.')
    @serial_test()
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_bucket_', additional_tags={'owner': 'cli_test'})
    def test_bucket_generate_akv_credential(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        volume_name = self.create_random_name(prefix='cli-vol-', length=24)
        bucket_name = self.create_random_name(prefix='clibkt', length=16)

        self.create_bucket(account_name, pool_name, volume_name, bucket_name, '{rg}')

        # generate akv credential - uses --bucket-name (no -n short form)
        self.cmd("az netappfiles volume bucket generate-akv-credential -g {rg} -a %s -p %s -v %s "
                 "--bucket-name %s --key-pair-expiry-days 3" %
                 (account_name, pool_name, volume_name, bucket_name))

    # NOTE: refresh-certificate requires Azure Key Vault certificate configuration on the bucket.
    # This test may need adjustment after live recording with a pre-configured AKV certificate.
    @unittest.skip('refresh-certificate requires Azure Key Vault certificate configuration on the bucket, which is not currently set up in the test environment. This test may need adjustment after live recording with a pre-configured AKV certificate.')
    @serial_test()
    @ResourceGroupPreparer(name_prefix='cli_netappfiles_test_bucket_', additional_tags={'owner': 'cli_test'})
    def test_bucket_refresh_certificate(self):
        account_name = self.create_random_name(prefix='cli-acc-', length=24)
        pool_name = self.create_random_name(prefix='cli-pool-', length=24)
        volume_name = self.create_random_name(prefix='cli-vol-', length=24)
        bucket_name = self.create_random_name(prefix='clibkt', length=16)

        self.create_bucket(account_name, pool_name, volume_name, bucket_name, '{rg}')

        # refresh certificate - uses --bucket-name (no -n short form)
        self.cmd("az netappfiles volume bucket refresh-certificate -g {rg} -a %s -p %s -v %s "
                 "--bucket-name %s" %
                 (account_name, pool_name, volume_name, bucket_name))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os

from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer)
from azure.cli.testsdk.scenario_tests import AllowLargeResponse

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))

class Cosmosdb_EnablePerRegionPerPartitionAutoscaleScenarioTest(ScenarioTest):

    @AllowLargeResponse()
    @ResourceGroupPreparer(name_prefix='cli_test_cosmosdb_sql_enable_per_region_per_partition_autoscale', location='australiaeast')
    def test_cosmosdb_enable_per_region_per_partition_autoscale(self, resource_group):
        col = self.create_random_name(prefix='cli', length=15)
        db_name = self.create_random_name(prefix='cli', length=15)
        # Assumption: There exists a cosmosTest rg.
        self.kwargs.update({
            'acc': self.create_random_name(prefix='prpp-test-', length=20),
            'loc': 'australiaeast',
            'tar': '0=1200 1=1200',
            'src': '2'
        })

        #create enablePerRegionPerPartitionAutoscale enabled account
        self.cmd('az cosmosdb create -n {acc} -g {rg} --enable-prpp-autoscale true')
        self.cmd('az cosmosdb show -n {acc} -g {rg}', checks=[
            self.check('enablePerRegionPerPartitionAutoscale', True),
        ])
        print('Created enablePerRegionPerPartitionAutoscale enabled account')

        #disable enablePerRegionPerPartitionAutoscale
        self.cmd('az cosmosdb update -n {acc} -g {rg} --enable-prpp-autoscale false')
        self.cmd('az cosmosdb show -n {acc} -g {rg}', checks=[
            self.check('enablePerRegionPerPartitionAutoscale', False),
        ])
        print('Disabled enablePerRegionPerPartitionAutoscale')

        #enable enablePerRegionPerPartitionAutoscale
        self.cmd('az cosmosdb update -n {acc} -g {rg} --enable-prpp-autoscale true')
        self.cmd('az cosmosdb show -n {acc} -g {rg}', checks=[
            self.check('enablePerRegionPerPartitionAutoscale', True),
        ])
        print('Enabled enablePerRegionPerPartitionAutoscale')

        #delete account
        self.cmd('az cosmosdb delete -n {acc} -g {rg} --yes')
        


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network watcher flow-log create",
)
class Create(AAZCommand):
    """Create a flow log on a network security group.

    :example: Create a flow log with Network Security Group name
        az network watcher flow-log create --location westus --resource-group MyResourceGroup --name MyFlowLog --nsg MyNetworkSecurityGroupName --storage-account account

    :example: Create a flow log with VNet name
        az network watcher flow-log create --location westus --resource-group MyResourceGroup --name MyFlowLog --vnet MyVNetName --storage-account account

    :example: Create a flow log with Subnet name
        az network watcher flow-log create --location westus --resource-group MyResourceGroup --name MyFlowLog --vnet MyVNetName --subnet MySubnetName --storage-account account

    :example: Create a flow log with NIC name
        az network watcher flow-log create --location westus --resource-group MyResourceGroup --name MyFlowLog --nic MyNICName --storage-account account

    :example: Create a flow log with Network Security Group ID (could be in other resource group)
        az network watcher flow-log create --location westus --name MyFlowLog --nsg MyNetworkSecurityGroupID --storage-account account

    :example: Create a flow log with Virtual Network ID (could be in other resource group)
        az network watcher flow-log create --location westus --name MyFlowLog --vnet MyVNetID --storage-account account

    :example: Create a flow log with Subnet ID (could be in other resource group)
        az network watcher flow-log create --location westus --name MyFlowLog --subnet SubnetID --storage-account account

    :example: Create a flow log with Network Interface ID (could be in other resource group)
        az network watcher flow-log create --location westus --name MyFlowLog --nic MyNetworkInterfaceID --storage-account account

    :example: Create or update flow log
        az network watcher flow-log create --location westus --resource-group MtRGContainingVNet --name MyVNetName-flowlog --vnet MyVNetName --storage-account MyStorageAccountName  --filtering-criteria "dstip=20.252.145.59 || DstPort=443"

    :example: Create flow log with recordtypes filtering
        az network watcher flow-log create --resource-group rg1 --network-watcher-name nw1 --name fl --location centraluseuap --target-resource-id /subscriptions/subid/resourceGroups/rg1/providers/Microsoft.Network/networkSecurityGroups/desmondcentral-nsg --storage-account /subscriptions/subid/resourceGroups/rg1/providers/Microsoft.Storage/storageAccounts/nwtest1mgvbfmqsigdxe --filtering-criteria srcIP=158.255.7.8 || dstPort=56891 --record-types B,E --enabled True --format JSON --log-version 1 --identity "{type:UserAssigned,user-assigned-identities:{/subscriptions/subid/resourceGroups/rg1/providers/Microsoft.ManagedIdentity/userAssignedIdentities/id1:{}}}"
    """

    _aaz_info = {
        "version": "2025-03-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/networkwatchers/{}/flowlogs/{}", "2025-03-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="The name of the flow logger.",
            required=True,
        )
        _args_schema.network_watcher_name = AAZStrArg(
            options=["--network-watcher-name"],
            help="The name of the network watcher.",
            required=True,
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.location = AAZResourceLocationArg(
            help="Location to identify the exclusive Network Watcher under a region. Only one Network Watcher can be existed per subscription and region.",
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.enabled = AAZBoolArg(
            options=["--enabled"],
            help="Enable logging. Default: true.",
        )
        _args_schema.storage_account = AAZStrArg(
            options=["--storage-account"],
            help="Name or ID of the storage account in which to save the flow logs.                                Must be in the same region of flow log.",
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            help="Space-separated tags: key[=value] [key[=value] ...]. Use \"\" to clear                                existing tags.",
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg()

        # define Arg Group "Format"

        _args_schema = cls._args_schema
        _args_schema.format = AAZStrArg(
            options=["--format"],
            arg_group="Format",
            help="File type of the flow log.",
            enum={"JSON": "JSON"},
        )
        _args_schema.log_version = AAZIntArg(
            options=["--log-version"],
            arg_group="Format",
            help="Version (revision) of the flow log.",
            default=0,
        )

        # define Arg Group "Parameters"

        _args_schema = cls._args_schema
        _args_schema.identity = AAZObjectArg(
            options=["--identity"],
            arg_group="Parameters",
            help="FlowLog resource Managed Identity",
        )

        identity = cls._args_schema.identity
        identity.mi_system_assigned = AAZStrArg(
            options=["system-assigned", "mi-system-assigned"],
            help="Set the system managed identity.",
            blank="True",
        )
        identity.type = AAZStrArg(
            options=["type"],
            help="The type of identity used for the resource. The type 'SystemAssigned, UserAssigned' includes both an implicitly created identity and a set of user assigned identities. The type 'None' will remove any identities from the virtual machine.",
            enum={"None": "None", "SystemAssigned": "SystemAssigned", "SystemAssigned, UserAssigned": "SystemAssigned, UserAssigned", "UserAssigned": "UserAssigned"},
        )
        identity.mi_user_assigned = AAZListArg(
            options=["user-assigned", "mi-user-assigned"],
            help="Set the user managed identities.",
            blank=[],
        )
        identity.user_assigned_identities = AAZDictArg(
            options=["user-assigned-identities"],
            help="The list of user identities associated with resource. The user identity dictionary key references will be ARM resource ids in the form: '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.",
        )

        mi_user_assigned = cls._args_schema.identity.mi_user_assigned
        mi_user_assigned.Element = AAZStrArg()

        user_assigned_identities = cls._args_schema.identity.user_assigned_identities
        user_assigned_identities.Element = AAZObjectArg(
            blank={},
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.filtering_criteria = AAZStrArg(
            options=["--filtering-criteria"],
            arg_group="Properties",
            help="Optional field to filter flowlogs based on SrcIP, SrcPort, DstIP, DstPort, Protocol, Encryption, Direction and Action. If not specified, all flowlogs will be logged.",
        )
        _args_schema.flow_analytics_configuration = AAZObjectArg(
            options=["--flow-analytics-configuration"],
            arg_group="Properties",
            help="Parameters that define the configuration of traffic analytics.",
        )
        _args_schema.record_types = AAZStrArg(
            options=["--record-types"],
            arg_group="Properties",
            help="Optional field to filter network traffic logs based on flow states. Value of this field could be any comma separated combination string of letters B,C,E or D. B represents Begin, when a flow is created. C represents Continue for an ongoing flow generated at every five-minute interval. E represents End, when a flow is terminated. D represents Deny, when a flow is denied. If not specified, all network traffic will be logged.",
        )
        _args_schema.retention_policy = AAZObjectArg(
            options=["--retention-policy"],
            arg_group="Properties",
            help="Parameters that define the retention policy for flow log.",
        )
        _args_schema.target_resource_id = AAZStrArg(
            options=["--target-resource-id"],
            arg_group="Properties",
            help="ID of network security group to which flow log will be applied.",
        )

        flow_analytics_configuration = cls._args_schema.flow_analytics_configuration
        flow_analytics_configuration.enabled = AAZBoolArg(
            options=["enabled"],
            help="Flag to enable/disable traffic analytics.",
        )
        flow_analytics_configuration.traffic_analytics_interval = AAZIntArg(
            options=["traffic-analytics-interval"],
            help="The interval in minutes which would decide how frequently TA service should do flow analytics.",
        )
        flow_analytics_configuration.workspace_id = AAZStrArg(
            options=["workspace-id"],
            help="The resource guid of the attached workspace.",
        )
        flow_analytics_configuration.workspace_region = AAZStrArg(
            options=["workspace-region"],
            help="The location of the attached workspace.",
        )
        flow_analytics_configuration.workspace_resource_id = AAZStrArg(
            options=["workspace-resource-id"],
            help="Resource Id of the attached workspace.",
        )

        retention_policy = cls._args_schema.retention_policy
        retention_policy.days = AAZIntArg(
            options=["days"],
            help="Number of days to retain flow log records.",
            default=0,
        )
        retention_policy.enabled = AAZBoolArg(
            options=["enabled"],
            help="Flag to enable/disable retention.",
            default=False,
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        yield self.FlowLogsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class FlowLogsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/networkWatchers/{networkWatcherName}/flowLogs/{flowLogName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "flowLogName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "networkWatcherName", self.ctx.args.network_watcher_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2025-03-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("identity", AAZIdentityObjectType, ".identity")
            _builder.set_prop("location", AAZStrType, ".location")
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            identity = _builder.get(".identity")
            if identity is not None:
                identity.set_prop("type", AAZStrType, ".type")
                identity.set_prop("userAssignedIdentities", AAZDictType, ".user_assigned_identities")
                identity.set_prop("userAssigned", AAZListType, ".mi_user_assigned", typ_kwargs={"flags": {"action": "create"}})
                identity.set_prop("systemAssigned", AAZStrType, ".mi_system_assigned", typ_kwargs={"flags": {"action": "create"}})

            user_assigned_identities = _builder.get(".identity.userAssignedIdentities")
            if user_assigned_identities is not None:
                user_assigned_identities.set_elements(AAZObjectType, ".")

            user_assigned = _builder.get(".identity.userAssigned")
            if user_assigned is not None:
                user_assigned.set_elements(AAZStrType, ".")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("enabled", AAZBoolType, ".enabled")
                properties.set_prop("enabledFilteringCriteria", AAZStrType, ".filtering_criteria")
                properties.set_prop("flowAnalyticsConfiguration", AAZObjectType, ".flow_analytics_configuration")
                properties.set_prop("format", AAZObjectType)
                properties.set_prop("recordTypes", AAZStrType, ".record_types")
                properties.set_prop("retentionPolicy", AAZObjectType, ".retention_policy")
                properties.set_prop("storageId", AAZStrType, ".storage_account", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("targetResourceId", AAZStrType, ".target_resource_id", typ_kwargs={"flags": {"required": True}})

            flow_analytics_configuration = _builder.get(".properties.flowAnalyticsConfiguration")
            if flow_analytics_configuration is not None:
                flow_analytics_configuration.set_prop("networkWatcherFlowAnalyticsConfiguration", AAZObjectType)

            network_watcher_flow_analytics_configuration = _builder.get(".properties.flowAnalyticsConfiguration.networkWatcherFlowAnalyticsConfiguration")
            if network_watcher_flow_analytics_configuration is not None:
                network_watcher_flow_analytics_configuration.set_prop("enabled", AAZBoolType, ".enabled")
                network_watcher_flow_analytics_configuration.set_prop("trafficAnalyticsInterval", AAZIntType, ".traffic_analytics_interval")
                network_watcher_flow_analytics_configuration.set_prop("workspaceId", AAZStrType, ".workspace_id")
                network_watcher_flow_analytics_configuration.set_prop("workspaceRegion", AAZStrType, ".workspace_region")
                network_watcher_flow_analytics_configuration.set_prop("workspaceResourceId", AAZStrType, ".workspace_resource_id")

            format = _builder.get(".properties.format")
            if format is not None:
                format.set_prop("type", AAZStrType, ".format")
                format.set_prop("version", AAZIntType, ".log_version")

            retention_policy = _builder.get(".properties.retentionPolicy")
            if retention_policy is not None:
                retention_policy.set_prop("days", AAZIntType, ".days")
                retention_policy.set_prop("enabled", AAZBoolType, ".enabled")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()

            _schema_on_200_201 = cls._schema_on_200_201
            _schema_on_200_201.etag = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.id = AAZStrType()
            _schema_on_200_201.identity = AAZIdentityObjectType()
            _schema_on_200_201.location = AAZStrType()
            _schema_on_200_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200_201.tags = AAZDictType()
            _schema_on_200_201.type = AAZStrType(
                flags={"read_only": True},
            )

            identity = cls._schema_on_200_201.identity
            identity.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )
            identity.tenant_id = AAZStrType(
                serialized_name="tenantId",
                flags={"read_only": True},
            )
            identity.type = AAZStrType()
            identity.user_assigned_identities = AAZDictType(
                serialized_name="userAssignedIdentities",
            )

            user_assigned_identities = cls._schema_on_200_201.identity.user_assigned_identities
            user_assigned_identities.Element = AAZObjectType()

            _element = cls._schema_on_200_201.identity.user_assigned_identities.Element
            _element.client_id = AAZStrType(
                serialized_name="clientId",
                flags={"read_only": True},
            )
            _element.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )

            properties = cls._schema_on_200_201.properties
            properties.enabled = AAZBoolType()
            properties.enabled_filtering_criteria = AAZStrType(
                serialized_name="enabledFilteringCriteria",
            )
            properties.flow_analytics_configuration = AAZObjectType(
                serialized_name="flowAnalyticsConfiguration",
            )
            properties.format = AAZObjectType()
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.record_types = AAZStrType(
                serialized_name="recordTypes",
            )
            properties.retention_policy = AAZObjectType(
                serialized_name="retentionPolicy",
            )
            properties.storage_id = AAZStrType(
                serialized_name="storageId",
                flags={"required": True},
            )
            properties.target_resource_guid = AAZStrType(
                serialized_name="targetResourceGuid",
                flags={"read_only": True},
            )
            properties.target_resource_id = AAZStrType(
                serialized_name="targetResourceId",
                flags={"required": True},
            )

            flow_analytics_configuration = cls._schema_on_200_201.properties.flow_analytics_configuration
            flow_analytics_configuration.network_watcher_flow_analytics_configuration = AAZObjectType(
                serialized_name="networkWatcherFlowAnalyticsConfiguration",
            )

            network_watcher_flow_analytics_configuration = cls._schema_on_200_201.properties.flow_analytics_configuration.network_watcher_flow_analytics_configuration
            network_watcher_flow_analytics_configuration.enabled = AAZBoolType()
            network_watcher_flow_analytics_configuration.traffic_analytics_interval = AAZIntType(
                serialized_name="trafficAnalyticsInterval",
            )
            network_watcher_flow_analytics_configuration.workspace_id = AAZStrType(
                serialized_name="workspaceId",
            )
            network_watcher_flow_analytics_configuration.workspace_region = AAZStrType(
                serialized_name="workspaceRegion",
            )
            network_watcher_flow_analytics_configuration.workspace_resource_id = AAZStrType(
                serialized_name="workspaceResourceId",
            )

            format = cls._schema_on_200_201.properties.format
            format.type = AAZStrType()
            format.version = AAZIntType()

            retention_policy = cls._schema_on_200_201.properties.retention_policy
            retention_policy.days = AAZIntType()
            retention_policy.enabled = AAZBoolType()

            tags = cls._schema_on_200_201.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200_201


class _CreateHelper:
    """Helper class for Create"""


__all__ = ["Create"]
